from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

from FeatureMap cimport *
from Feature cimport *
from Param cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinder.h>" namespace "OpenMS":

    cdef cppclass FeatureFinder(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger
        #

        FeatureFinder()      nogil except +
        void run(String algorithm_name,
                 MSExperiment & input_map,
                 FeatureMap & feats,
                 Param & param,
                 FeatureMap & seeds
                 ) nogil except +

        Param getParameters(String algorithm_name) nogil except +



from Feature cimport *
from FeatureMap cimport *
from String cimport *
from Types cimport *

cdef extern from "<OpenMS/METADATA/AbsoluteQuantitationStandards.h>" namespace "OpenMS":

    cdef cppclass AbsoluteQuantitationStandards:

        AbsoluteQuantitationStandards() nogil except +
        AbsoluteQuantitationStandards(AbsoluteQuantitationStandards) nogil except +

        void getComponentFeatureConcentrations(
            libcpp_vector[AQS_runConcentration]& run_concentrations,
            libcpp_vector[FeatureMap]& feature_maps,
            const String& component_name,
            libcpp_vector[AQS_featureConcentration]& feature_concentrations
        ) nogil except +

        # void mapComponentsToConcentrations(libcpp_vector[ AQS_runConcentration ] run_concentrations,
        #                                    libcpp_vector[ FeatureMap ] feature_maps,
        #                                    libcpp_map[ String, libcpp_vector[ AQS_featureConcentration ]] & components_to_concentrations) nogil except +

cdef extern from "<OpenMS/METADATA/AbsoluteQuantitationStandards.h>" namespace "OpenMS::AbsoluteQuantitationStandards":

    cdef cppclass AQS_runConcentration "OpenMS::AbsoluteQuantitationStandards::runConcentration":

        AQS_runConcentration() nogil except +
        AQS_runConcentration(AQS_runConcentration) nogil except +

        String sample_name
        String component_name
        String IS_component_name
        double actual_concentration
        double IS_actual_concentration
        String concentration_units
        double dilution_factor

    cdef cppclass AQS_featureConcentration "OpenMS::AbsoluteQuantitationStandards::featureConcentration":

        AQS_featureConcentration() nogil except +
        AQS_featureConcentration(AQS_featureConcentration) nogil except +

        Feature feature
        Feature IS_feature
        double actual_concentration
        double IS_actual_concentration
        String concentration_units
        double dilution_factor






    def convertToSpectrumPtr(self, MSSpectrum spectrum):
        assert isinstance(spectrum, MSSpectrum), 'arg spec wrong type'

        _r = self.inst.get().convertToSpectrumPtr((deref(spectrum.inst.get())))
        # cdef shared_ptr[_OSSpectrum] py_result = _r
        spec = OSSpectrum()
        spec.inst = _r
        return spec

    def convertToChromatogramPtr(self, MSChromatogram chrom):
        assert isinstance(chrom, MSChromatogram), 'arg spec wrong type'

        _r = self.inst.get().convertToChromatogramPtr((deref(chrom.inst.get())))
        # cdef shared_ptr[_OSChromatogram] py_result = _r
        ch = OSChromatogram()
        ch.inst = _r
        return ch




from Types cimport *
from MSExperiment cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerSH.h>" namespace "OpenMS":
    
    cdef cppclass PeakPickerSH(DefaultParamHandler,ProgressLogger) :
        # wrap-inherits:
        #  DefaultParamHandler
        #  ProgressLogger
        PeakPickerSH() nogil except +
        PeakPickerSH(PeakPickerSH) nogil except + #wrap-ignore
        void pick(MSSpectrum & input_, MSSpectrum & output, float fWindowWidth) nogil except +
        void pickExperiment(MSExperiment & input_, MSExperiment & output) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from TargetedExperiment cimport *

# typedef boost::unordered_map<String, double> IonSeries;

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMIonSeries.h>" namespace "OpenMS":
    
    cdef cppclass MRMIonSeries "OpenMS::MRMIonSeries":
        MRMIonSeries() nogil except +
        MRMIonSeries(MRMIonSeries) nogil except + #wrap-ignore

        ## Typedef boost::unordered_map
        ## libcpp_pair[ String, double ] getIon(IonSeries ionseries, String ionid) nogil except +
        ## libcpp_pair[ String, double ] annotateIon(IonSeries ionseries, double ProductMZ, double mz_threshold) nogil except +

        void annotateTransitionCV(ReactionMonitoringTransition & tr, String annotation) nogil except +

        void annotateTransition(ReactionMonitoringTransition & tr,
                                Peptide peptide, double
                                precursor_mz_threshold, double
                                product_mz_threshold, bool enable_reannotation,
                                libcpp_vector[ String ] fragment_types,
                                libcpp_vector[ size_t ] fragment_charges, 
                                bool enable_specific_losses, 
                                bool enable_unspecific_losses, int round_decPow) nogil except +

        ## IonSeries getIonSeries(AASequence sequence, size_t precursor_charge,
        ##                        libcpp_vector[ String ] fragment_types,
        ##                        libcpp_vector[ size_t ] fragment_charges, 
        ##                        bool enable_specific_losses,
        ##                        bool enable_unspecific_losses, int round_decPow) nogil except +






    def create(self):
        cdef ComplementMarker res = ComplementMarker.__new__(ComplementMarker)
        res.inst = shared_ptr[_ComplementMarker]( <_ComplementMarker *> self.inst.get().create() )
        return res



from Types cimport *
from IsobaricQuantitationMethod cimport *
from ItraqFourPlexQuantitationMethod cimport *
from ItraqEightPlexQuantitationMethod cimport *
from TMTSixPlexQuantitationMethod cimport *
from TMTTenPlexQuantitationMethod cimport *
from IsobaricQuantifierStatistics cimport *
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricNormalizer.h>" namespace "OpenMS":
    
    cdef cppclass IsobaricNormalizer "OpenMS::IsobaricNormalizer":
        IsobaricNormalizer(IsobaricNormalizer) nogil except +

        IsobaricNormalizer(IsobaricQuantitationMethod *quant_method) nogil except + # wrap-ignore
        IsobaricNormalizer(ItraqFourPlexQuantitationMethod *quant_method) nogil except +
        IsobaricNormalizer(ItraqEightPlexQuantitationMethod *quant_method) nogil except +
        IsobaricNormalizer(TMTSixPlexQuantitationMethod *quant_method) nogil except +
        IsobaricNormalizer(TMTTenPlexQuantitationMethod *quant_method) nogil except +

        void normalize(ConsensusMap & consensus_map) nogil except +




from InterpolationModel cimport *
from String cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/EmgModel.h>" namespace "OpenMS":
    
    cdef cppclass EmgModel(InterpolationModel):
        # wrap-inherits:
        #  InterpolationModel
        EmgModel() nogil except +
        EmgModel(EmgModel) nogil except +
        # BaseModel[ 1 ] * create() nogil except +
        String getProductName() nogil except +

        # inherited from parent class - no second definition necessary!
        # void setOffset(CoordinateType offset) # wrap-ignore
        # void setSamples() # wrap-ignore
        # CoordinateType getCenter() # wrap-ignore




import sys

if sys.platform.startswith("linux"):
    import ctypes as c
    import ctypes.util

    libc = c.CDLL(ctypes.util.find_library("c"))

    class SysInfo(c.Structure):

         # for libc5: needs padding
        padding = 20 - 2 * c.sizeof(c.c_long) - c.sizeof(c.c_int)

        _fields_ = [("uptime", c.c_ulong),
                    ("loads", 3 * c.c_ulong),
                    ("totalram", c.c_ulong),
                    ("freeram", c.c_ulong),
                    ("sharedram", c.c_ulong),
                    ("bufferram", c.c_ulong),
                    ("totalswap", c.c_ulong),
                    ("freeswap", c.c_ulong),
                    ("procs", c.c_ushort),
                    ("totalhigh", c.c_ulong),
                    ("freehigh", c.c_ulong),
                    ("mem_unit", c.c_ulong),
                    ("_padding", padding * c.c_char)
        ]

    def free_mem():
        sys_info = SysInfo()
        libc.sysinfo(c.byref(sys_info))
        return sys_info.freeram

elif sys.platform == "win32":
    try:
        import win32api
    except:
        free_mem = lambda: 0 # memory will never change !
    else:
        def free_mem():
            return win32api.GlobalMemoryStatus()['AvailPhys']

else:
    sys.stderr.write("Determination of memory status is not supported on this \n"
                     " platform, measuring for memoryleaks will never fail\n")

    free_mem = lambda: 0 # memory will never change !



from Types cimport *
from String cimport *
from MSSpectrum cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/SiriusFragmentAnnotation.h>" namespace "OpenMS":

    cdef cppclass SiriusFragmentAnnotation:
        SiriusFragmentAnnotation() nogil except +
        SiriusFragmentAnnotation(SiriusFragmentAnnotation) nogil except +
                
        void  extractSiriusFragmentAnnotationMapping(String& path_to_sirius_workspace,
                                                     MSSpectrum& msspectrum_to_fill,
                                                     bool use_exact_mass) nogil except +


from libcpp cimport *
from Types cimport *
from String cimport *
from ProteinIdentification cimport *
from ConsensusMap cimport *
from FeatureMap cimport *

cdef extern from "<OpenMS/METADATA/ExperimentalDesign.h>" namespace "OpenMS":
    
    cdef cppclass ExperimentalDesign "OpenMS::ExperimentalDesign":
        ExperimentalDesign() nogil except +
        ExperimentalDesign(ExperimentalDesign) nogil except + #wrap-ignore

        libcpp_vector[ ExperimentalDesign_MSFileSectionEntry ] getMSFileSection() nogil except +
        void setMSFileSection(libcpp_vector[ ExperimentalDesign_MSFileSectionEntry ] msfile_section) nogil except +

        # Returns the Sample Section of the experimental design file
        ExperimentalDesign_SampleSection getSampleSection() nogil except +
        void setSampleSection(ExperimentalDesign_SampleSection sample_section) nogil except +

        # fraction index to file paths (ordered by fraction_group)
        #lib_map[unsigned int, lib_cpp[ String ] ] getFractionToMSFilesMapping() nogil except +
     
        # return <file_path, label> to sample mapping
        #lib_map[ std::pair< String, unsigned >, unsigned] getPathLabelToSampleMapping(bool) nogil except +

        # return <file_path, label> to fraction mapping
        #std::map< std::pair< String, unsigned >, unsigned> getPathLabelToFractionMapping(bool) const;

        # return <file_path, label> to fraction_group mapping
        #std::map< std::pair< String, unsigned >, unsigned> getPathLabelToFractionGroupMapping(bool) const;

        # @return the number of samples measured (= highest sample index)
        unsigned int getNumberOfSamples() nogil except +

        # @return the number of fractions (= highest fraction index)
        unsigned int getNumberOfFractions() nogil except +

        # @return the number of labels per file
        unsigned int getNumberOfLabels() nogil except +

        # @return the number of MS files (= fractions * fraction_groups)
        unsigned int getNumberOfMSFiles() nogil except +

        # @return the number of fraction_groups
        # Allows to group fraction ids and source files
        unsigned int getNumberOfFractionGroups() nogil except +

        # @return sample index (depends on fraction_group and label)
        unsigned int getSample(unsigned int fraction_group, unsigned int label) nogil except +

        # @return whether at least one fraction_group in this experimental design is fractionated
        bool isFractionated() nogil except +

        # return if each fraction number is associated with the same number of fraction_group
        bool sameNrOfMSFilesPerFraction() nogil except +
                
# COMMENT: wrap static methods
cdef extern from "<OpenMS/METADATA/ExperimentalDesign.h>" namespace "OpenMS::ExperimentalDesign":

        # Extract experimental design from consensus map
        ExperimentalDesign fromConsensusMap(ConsensusMap c) nogil except + #wrap-attach:ExperimentalDesign

        # Extract experimental design from feature map
        ExperimentalDesign fromFeatureMap(FeatureMap f) nogil except + #wrap-attach:ExperimentalDesign

        # Extract experimental design from identifications
        ExperimentalDesign fromIdentifications(libcpp_vector[ProteinIdentification] & proteins) nogil except + #wrap-attach:ExperimentalDesign

cdef extern from "<OpenMS/METADATA/ExperimentalDesign.h>" namespace "OpenMS::ExperimentalDesign":
    
    cdef cppclass ExperimentalDesign_MSFileSectionEntry "OpenMS::ExperimentalDesign::MSFileSectionEntry":

        ExperimentalDesign_MSFileSectionEntry() nogil except +
        ExperimentalDesign_MSFileSectionEntry(ExperimentalDesign_MSFileSectionEntry) nogil except + #wrap-ignore

        libcpp_string path
        unsigned int fraction_group
        unsigned int fraction
        unsigned int label
        unsigned int sample
        
cdef extern from "<OpenMS/METADATA/ExperimentalDesign.h>" namespace "OpenMS::ExperimentalDesign":

    cdef cppclass ExperimentalDesign_SampleSection "OpenMS::ExperimentalDesign::SampleSection":

          ExperimentalDesign_SampleSection() nogil except +
          ExperimentalDesign_SampleSection(ExperimentalDesign_SampleSection) nogil except +

          # ExperimentalDesign_SampleSection(const libcpp_vector[ libcpp_vector[ String ] ] & content,
          #                                  libcpp_map[ unsigned, size_t ] sample_to_rowindex, 
          #                                  libcpp_map[ String, size_t ] columnname_to_columnindex) nogil except +

          # Get set of all samples that are present in the sample section
          libcpp_set[ unsigned int ] getSamples() nogil except +

          # Get set of all factors (column names) that were defined for the sample section
          libcpp_set[ String ] getFactors() nogil except +

          # Checks whether sample section has row for a sample number
          bool hasSample(unsigned int sample) nogil except +

          # Checks whether Sample Section has a specific factor (i.e. column name)
          bool hasFactor(String& factor) nogil except +

          # Returns value of factor for given sample and factor name
          String getFactorValue(unsigned int sample, String &factor) nogil except +
                   




from Types cimport *
from DataValue cimport *
from String cimport *
from StringList cimport *
from ParamNode cimport *
from ParamEntry cimport *

# this class has addons, see the ./addons folder

cdef extern from "<OpenMS/DATASTRUCTURES/Param.h>" namespace "OpenMS":

    # pythonic helper functions in ../addons/Param.pyx !!!!

    cdef cppclass Param:

         # COMMENT: Helper functions for Python
         asDict() # wrap-ignore
         keys() # wrap-ignore
         items() # wrap-ignore
         values() # wrap-ignore
         update(dict) # wrap-ignore
         get(bytes key, default=None) # wrap-ignore
         __getitem__(bytes key) # wrap-ignore
         __setitem__(bytes key, value) # wrap-ignore

         Param() nogil except +
         Param(Param) nogil except +
         bool operator==(Param) nogil except +

         void setValue(String key, DataValue val, String desc, StringList tags) nogil except +
         void setValue(String key, DataValue val, String desc) nogil except +
         void setValue(String key, DataValue val) nogil except +
         DataValue getValue(String key) nogil except +
         ParamEntry getEntry(String) nogil except +
         bool exists(String key) nogil except +

         void addTag(String key, String tag) nogil except +
         void addTags(String key, StringList tags) nogil except +
         int hasTag(String key, String tag) nogil except +
         StringList getTags(String key) nogil except +
         void clearTags(String key) nogil except +

         String getDescription(String key) nogil except +
         void setSectionDescription(String key, String desc) nogil except +
         String getSectionDescription(String key) nogil except +

         Size size() nogil except +
         bool empty() nogil except +

         void clear() nogil except +
         void insert(String prefix, Param param) nogil except +

         void remove(String key) nogil except +
         void removeAll(String prefix) nogil except +

         Param copy(String prefix, bool) nogil except +
         Param copy(String prefix) nogil except +

         # wrapped manually for overloading with dict parameter:
         bool update(Param p_old, bool add_unknow) nogil except + # wrap-ignore
         bool update(Param p_old) nogil except + # wrap-ignore

         void merge(Param toMerge) nogil except +

         void setDefaults(Param defaults, String previx, bool showMessage) nogil except +
         void setDefaults(Param defaults, String previx) nogil except +
         void setDefaults(Param defaults) nogil except +

         void checkDefaults(String name, Param defaults, String prefix) nogil except +
         void checkDefaults(String name, Param defaults) nogil except +

         void setValidStrings(String key, libcpp_vector[String] strings) nogil except +
         void setMinInt(String key, int min) nogil except +
         void setMaxInt(String key, int max) nogil except +
         void setMinFloat(String key, double min) nogil except +
         void setMaxFloat(String key, double max) nogil except +

         #void parseCommandLine(int argc, char ** argv, String prefix) # wrap-ignore
         #void parseCommandLine(int argc, char ** argv) # wrap-ignore

         ParamIterator begin() nogil except + # wrap-ignore
         ParamIterator end()   nogil except + # wrap-ignore

cdef extern from "<OpenMS/DATASTRUCTURES/Param.h>" namespace "OpenMS::Param":

    cdef cppclass ParamIterator:
        # wrap-ignore
        # no-pxd-import
        ParamIterator operator++() nogil except +
        ParamIterator operator--() nogil except +
        String getName() nogil except +
        int operator==(ParamIterator) nogil except +
        int operator!=(ParamIterator) nogil except +
        int operator<(ParamIterator) nogil except +
        int operator>(ParamIterator) nogil except +
        int operator<=(ParamIterator) nogil except +
        int operator>=(ParamIterator) nogil except +

        # Returns the traceback of the opened and closed sections
        libcpp_vector[TraceInfo] getTrace() nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/Param.h>" namespace "OpenMS::Param::ParamIterator":

    cdef cppclass TraceInfo:

        TraceInfo(String n, String d, bool o) nogil except +
        TraceInfo(TraceInfo) nogil except +

        # name of the node
        String name
        # description of the node
        String description
        # If it was opened (true) or closed (false)
        bool opened



from Types cimport *
from String cimport *
from DigestionEnzymeRNA cimport *

cdef extern from "<OpenMS/CHEMISTRY/RNaseDB.h>" namespace "OpenMS":

    cdef cppclass RNaseDB "OpenMS::RNaseDB":
        # wrap-manual-memory:
        #     cdef AutowrapPtrHolder[_RNaseDB] inst

        RNaseDB() nogil except + #wrap-ignore
        RNaseDB(RNaseDB) nogil except + #wrap-ignore

        const DigestionEnzymeRNA* getEnzyme(const String& name) nogil except +
        const DigestionEnzymeRNA* getEnzymeByRegEx(const String& cleavage_regex) nogil except +
        void getAllNames(libcpp_vector[ String ]& all_names) nogil except +
        bool hasEnzyme(const String& name) nogil except +
        bool hasRegEx(const String& cleavage_regex) nogil except +
        # bool hasEnzyme(DigestionEnzymeRNA* enzyme) nogil except + # does not make sense as the ptr wont match

        # ConstEnzymeIterator beginEnzyme() nogil except +
        # ConstEnzymeIterator endEnzyme() nogil except +

## wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/RNaseDB.h>" namespace "OpenMS::RNaseDB":

    RNaseDB* getInstance() nogil except + # wrap-ignore



import argparse
import pyopenms as pms
from common import addDataProcessing, writeParamsIfRequested, updateDefaults


def align(in_files, out_files, out_trafos, reference_index,
          reference_file, params):

    in_types = set(pms.FileHandler.getType(in_) for in_ in in_files)

    if in_types <= set((pms.Type.MZML, pms.Type.MZXML, pms.Type.MZDATA)):
        align_features = False
    elif in_types == set((pms.Type.FEATUREXML,)):
        align_features = True
    else:
        raise Exception("different kinds of input files")

    algorithm = pms.MapAlignmentAlgorithmPoseClustering()
    alignment_params = params.copy("algorithm:", True)
    algorithm.setParameters(alignment_params)
    algorithm.setLogType(pms.LogType.CMD)

    plog = pms.ProgressLogger()
    plog.setLogType(pms.LogType.CMD)

    if reference_file:
        file_ = reference_file
    elif reference_index > 0:
        file_ = in_files[reference_index-1]
    else:
        sizes = []
        if align_features:
            fh = pms.FeatureXMLFile()
            plog.startProgress(0, len(in_files), "Determine Reference map")
            for i, in_f in enumerate(in_files):
                sizes.append((fh.loadSize(in_f), in_f))
                plog.setProgress(i)
        else:
            fh = pms.MzMLFile()
            mse = pms.MSExperiment()
            plog.startProgress(0, len(in_files), "Determine Reference map")
            for i, in_f in enumerate(in_files):
                fh.load(in_f, mse)
                mse.updateRanges()
                sizes.append((mse.getSize(), in_f))
                plog.setProgress(i)
        plog.endProgress()
        __, file_ = max(sizes)

    f_fmxl = pms.FeatureXMLFile()
    if not out_files:
        options = f_fmxl.getOptions()
        options.setLoadConvexHull(False)
        options.setLoadSubordinates(False)
        f_fmxl.setOptions(options)

    if align_features:
        map_ref = pms.FeatureMap()
        f_fxml_tmp = pms.FeatureXMLFile()
        options = f_fmxl.getOptions()
        options.setLoadConvexHull(False)
        options.setLoadSubordinates(False)
        f_fxml_tmp.setOptions(options)
        f_fxml_tmp.load(file_, map_ref)
        algorithm.setReference(map_ref)
    else:
        map_ref = pms.MSExperiment()
        pms.MzMLFile().load(file_, map_ref)
        algorithm.setReference(map_ref)

    plog.startProgress(0, len(in_files), "Align input maps")
    for i, in_file in enumerate(in_files):
        trafo = pms.TransformationDescription()
        if align_features:
            map_ = pms.FeatureMap()
            f_fxml_tmp = pms.FeatureXMLFile()
            f_fxml_tmp.setOptions(f_fmxl.getOptions())
            f_fxml_tmp.load(in_file, map_)
            if in_file == file_:
                trafo.fitModel("identity")
            else:
                algorithm.align(map_, trafo)
            if out_files:
                pms.MapAlignmentTransformer.transformRetentionTimes(map_, trafo)
                addDataProcessing(map_, params, pms.ProcessingAction.ALIGNMENT)
                f_fxml_tmp.store(out_files[i], map_)
        else:
            map_ = pms.MSExperiment()
            pms.MzMLFile().load(in_file, map_)
            if in_file == file_:
                trafo.fitModel("identity")
            else:
                algorithm.align(map_, trafo)
            if out_files:
                pms.MapAlignmentTransformer.transformRetentionTimes(map_, trafo)
                addDataProcessing(map_, params, pms.ProcessingAction.ALIGNMENT)
                pms.MzMLFile().store(out_files[i], map_)
        if out_trafos:
            pms.TransformationXMLFile().store(out_trafos[i], trafo)

        plog.setProgress(i+1)

    plog.endProgress()



def getModelDefaults(default_model):
    params = pms.Param()
    params.setValue("type", default_model, "Type of model")
    model_types = [ "linear", "interpolated"]
    if default_model not in model_types:
        model_types.insert(0, default_model)
    params.setValidStrings("type", model_types)

    model_params = pms.Param()

    pms.TransformationModelLinear.getDefaultParameters(model_params)
    params.insert("linear:", model_params)
    params.setSectionDescription("linear", "Parameters for 'linear' model")

    pms.TransformationModelInterpolated.getDefaultParameters(model_params)
    entry = model_params.getEntry("interpolation_type")
    interpolation_types = entry.valid_strings
    model_params.setValidStrings("interpolation_type", interpolation_types)

    params.insert("interpolated:", model_params)
    params.setSectionDescription("interpolated", "Parameters for 'interpolated' model")
    return params


def getDefaultParameters():
    model_param = getModelDefaults("linear")
    algo_param = pms.MapAlignmentAlgorithmPoseClustering().getParameters()
    default = pms.Param()
    default.insert("model:", model_param)
    default.insert("algorithm:", algo_param)
    return default


def main():

    parser = argparse.ArgumentParser(description="PeakPickerHiRes")
    parser.add_argument("-in",
                        action="append",
                        type=str,
                        dest="in_",
                        metavar="input_file",
                        )

    parser.add_argument("-seeds",
                        action="store",
                        type=str,
                        metavar="seeds_file",
                        )

    parser.add_argument("-out",
                        action="append",
                        type=str,
                        metavar="output_file",
                        )

    parser.add_argument("-trafo_out",
                        action="append",
                        type=str,
                        metavar="output_file",
                        )

    parser.add_argument("-ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-write_ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-write_dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-reference:file",
                        action="store",
                        type=str,
                        metavar="reference_file",
                        dest="reference_file",
                        )
    parser.add_argument("-reference:index",
                        action="store",
                        type=int,
                        metavar="reference_index",
                        dest="reference_index",
                        )

    args = parser.parse_args()

    def collect(args):
        return [f.strip() for arg in args or [] for f in arg.split(",")]

    in_files = collect(args.in_)
    out_files = collect(args.out)
    trafo_out_files = collect(args.trafo_out)

    run_mode = (in_files and (out_files or trafo_out_files))\
                and (args.ini is not None or args.dict_ini is not None)

    write_mode = args.write_ini is not None or args.write_dict_ini is not None
    ok = run_mode or write_mode
    if not ok:
        parser.error("either specify -in, -(trafo_)out and -(dict)ini for running "
                     "the map aligner\nor -write(dict)ini for creating std "
                     "ini file")

    defaults = getDefaultParameters()
    write_requested = writeParamsIfRequested(args, defaults)

    if not write_requested:
        updateDefaults(args, defaults)

        if not out_files and not trafo_out_files:
            parser.error("need -out or -trafo_out files")

        if out_files and len(out_files) != len(in_files):
            parser.error("need as many -out files as -in files")
        if trafo_out_files and len(trafo_out_files) != len(in_files):
            parser.error("need as many -trafo_out files as -in files")

        if args.reference_index is not None and args.reference_file is not None:
            parser.error("can only handle either reference:index or reference:file")

        if args.reference_index is not None:
            if args.reference_index <0 or args.reference_index >= len(in_files):
                parser.error("reference:index invalid")
        if args.reference_file is not None:
            if args.reference_file not in in_files:
                parser.error("reference_file not in input files")


        align(in_files, out_files, trafo_out_files, args.reference_index or 0,
                args.reference_file or "", defaults)



if __name__ == "__main__":
    main()



from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DataValue cimport *
from String cimport *
from Types cimport *
from MetaInfoDescription cimport *

# see ../addons/StringDataArray.pyx

cdef extern from "<OpenMS/METADATA/DataArrays.h>" namespace "OpenMS::DataArrays":

    cdef cppclass StringDataArray(MetaInfoDescription):
        # wrap-inherits:
        #  MetaInfoDescription
        #
        # wrap-doc:
        #   The representation of extra string data attached to a spectrum or chromatogram.

        StringDataArray() nogil except +
        StringDataArray(StringDataArray) nogil except + #wrap-ignore

        Size size() nogil except +
        void resize(size_t n) nogil except +
        String& operator[](int) nogil except + # wrap-ignore
        void clear() nogil except +
        void push_back(String) nogil except +





from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from Types cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/DistanceMatrix.h>" namespace "OpenMS":
    
    cdef cppclass DistanceMatrix[Value]:
        # wrap-instances:
        #   DistanceMatrix := DistanceMatrix[float]
        DistanceMatrix() nogil except +
        DistanceMatrix(DistanceMatrix) nogil except +
        DistanceMatrix(size_t dimensionsize, Value value) nogil except +

        # ValueType operator()(size_t i, size_t j) nogil except +
        # ValueType operator()(size_t i, size_t j) nogil except +
        Value getValue(size_t i, size_t j) nogil except +
        void setValue(size_t i, size_t j, Value value) nogil except +
        void setValueQuick(size_t i, size_t j, Value value) nogil except +
        void clear() nogil except +
        void resize(size_t dimensionsize, Value value) nogil except +
        void reduce(size_t j) nogil except +
        size_t dimensionsize() nogil except +
        void updateMinElement() nogil except +
        bool operator==(DistanceMatrix[ Value ] &rhs) nogil except +
        libcpp_pair[ size_t, size_t ] getMinElementCoordinates() nogil except +
        # TEMPLATE # std::ostream  operator[[(std::ostream &os, DistanceMatrix[ Value ] &matrix) nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from String cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/CVReference.h>" namespace "OpenMS":
    
    cdef cppclass CVReference "OpenMS::CVReference":
        CVReference() nogil except +
        CVReference(CVReference) nogil except +
        void setName(const String &name) nogil except +
        String getName() nogil except +
        void setIdentifier(const String &identifier) nogil except +
        String getIdentifier() nogil except +
        bool operator==(CVReference &rhs) nogil except +
        bool operator!=(CVReference &rhs) nogil except +




from Types cimport *
from libcpp cimport bool
#from PeakSpectrumCompareFunctor cimport *
from BinnedSpectrum cimport *
from String cimport *
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/COMPARISON/SPECTRA/SpectraSTSimilarityScore.h>" namespace "OpenMS":
    
    cdef cppclass SpectraSTSimilarityScore:
        #  PeakSpectrumCompareFunctor inheritance
        SpectraSTSimilarityScore() nogil except +
        SpectraSTSimilarityScore(SpectraSTSimilarityScore) nogil except +
        # TODO operator ()
        # double operator()(MSSpectrum & spec1, MSSpectrum & spec2) nogil except +
        # double operator()(BinnedSpectrum & bin1, BinnedSpectrum & bin2) nogil except +
        # double operator()(MSSpectrum & spec) nogil except +
        bool preprocess(MSSpectrum & spec, float remove_peak_intensity_threshold, UInt cut_peaks_below, Size min_peak_number, Size max_peak_number) nogil except +
        BinnedSpectrum transform(MSSpectrum & spec) nogil except +
        double dot_bias(BinnedSpectrum & bin1, BinnedSpectrum & bin2, double dot_product) nogil except +
        double delta_D(double top_hit, double runner_up) nogil except +
        double compute_F(double dot_product, double delta_D, double dot_bias) nogil except +
        # POINTER # MSSpectrumCompareFunctor * create() nogil except +
        String getProductName() nogil except +




# from MaxLikeliFitter1D cimport *
# from BasicStatistics cimport *
from String cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/BiGaussFitter1D.h>" namespace "OpenMS":
    
    cdef cppclass BiGaussFitter1D "OpenMS::BiGaussFitter1D":
        BiGaussFitter1D() nogil except +
        BiGaussFitter1D(BiGaussFitter1D) nogil except +
        # QualityType fit1d(RawDataArrayType &range, InterpolationModel *&model) nogil except +
        # Fitter1D * create() nogil except +
        String getProductName() nogil except +




from ExperimentalSettings cimport *
from Types cimport *

cdef extern from "<OpenMS/INTERFACES/IMSDataConsumer.h>" namespace "OpenMS::Interfaces":

    cdef cppclass IMSDataConsumer[SpectrumType, ChromatogramType]:
        # wrap-ignore
        # no-pxd-import
        # ABSTRACT class

        void consumeSpectrum(SpectrumType &)

        void consumeChromatogram(ChromatogramType &)

        void setExpectedSize(Size, Size)

        void setExperimentalSettings(ExperimentalSettings &)




import argparse
import pyopenms as pms
from common import addDataProcessing, writeParamsIfRequested, updateDefaults


def id_mapper(in_file, id_file, out_file, params, use_centroid_rt,
        use_centroid_mz, use_subelements ):

    in_type = pms.FileHandler.getType(in_file)

    protein_ids = []
    peptide_ids = []

    pms.IdXMLFile().load(id_file, protein_ids, peptide_ids)

    mapper = pms.IDMapper()
    mapper.setParameters(params)

    if in_type == pms.Type.CONSENSUSXML:
        file_ = pms.ConsensusXMLFile()
        map_ = pms.ConsensusMap()
        file_.load(in_file, map_)
        mapper.annotate(map_, peptide_ids, protein_ids, use_subelements)
        addDataProcessing(map_, params, pms.ProcessingAction.IDENTIFICATION_MAPPING)
        file_.store(out_file, map_)

    elif in_type == pms.Type.FEATUREXML:
        file_ = pms.FeatureXMLFile()
        map_ = pms.FeatureMap()
        file_.load(in_file, map_)
        mapper.annotate(map_, peptide_ids, protein_ids, use_centroid_rt,
                use_centroid_mz)
        addDataProcessing(map_, params, pms.ProcessingAction.IDENTIFICATION_MAPPING)
        file_.store(out_file, map_)

    elif in_type == pms.Type.MZQ:
        file_ = pms.MzQuantMLFile()
        msq = pms.MSQuantifications()
        file_.load(in_file, msq)
        maps = msq.getConsensusMaps()
        for map_ in maps:
            mapper.annotate(map_, peptide_ids, protein_ids, use_subelements)
            addDataProcessing(map_, params, pms.ProcessingAction.IDENTIFICATION_MAPPING)
        msq.setConsensusMaps(maps)
        file_.store(out_file, msq)

    else:
        raise Exception("invalid input file format")


def main():

    parser = argparse.ArgumentParser(description="IDMapper")

    parser.add_argument("-id",
                        action="store",
                        type=str,
                        dest="id_",
                        metavar="id_file",
                        )

    parser.add_argument("-in",
                        action="store",
                        type=str,
                        dest="in_",
                        metavar="input_file",
                        )

    parser.add_argument("-out",
                        action="store",
                        type=str,
                        metavar="output_file",
                        )

    parser.add_argument("-ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-write_ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-write_dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-mz_tolerance",
                        action="store",
                        type=float)

    parser.add_argument("-rt_tolerance",
                        action="store",
                        type=float)

    parser.add_argument("-mz_measure",
                        action="store",
                        type=str)

    parser.add_argument("-mz_reference",
                        action="store",
                        type=str)

    parser.add_argument("-ignore_charge",
                        action="store_true")

    parser.add_argument("-feature:use_centroid_rt",
                        action="store_true")

    parser.add_argument("-feature:use_centroid_mz",
                        action="store_true")

    parser.add_argument("-consensusfeature:use_subelements",
                        action="store_true")

    args = parser.parse_args()



    run_mode = (args.in_ and args.id_ and args.out) \
                and (args.ini is not None or args.dict_ini is not None)

    write_mode = args.write_ini is not None or args.write_dict_ini is not None
    ok = run_mode or write_mode
    if not ok:
        parser.error("either specify -id, -in, -out and -(dict)ini for running "
                     "the id mapper\nor -write(dict)ini for creating std "
                     "ini file")

    defaults = pms.IDMapper().getParameters()
    write_requested = writeParamsIfRequested(args, defaults)

    if not write_requested:
        updateDefaults(args, defaults)

        dd = defaults.asDict()
        dd["ignore_charge"] = "true" if args.ignore_charge else "false"
        for att in ["mz_tolerance", "rt_tolerance", "mz_measure",
                "mz_reference"]:
            value = getattr(args, att)
            if value is not None:
                dd[att] = value

        defaults.update(dd)
        feature_use_centroid_rt = getattr(args, "feature:use_centroid_rt")
        feature_use_centroid_mz = getattr(args, "feature:use_centroid_mz")
        consenususfeature_use_subelements = getattr(args,
                "consensusfeature:use_subelements")

        id_mapper(args.in_, args.id_, args.out, defaults,
                feature_use_centroid_rt,
                feature_use_centroid_mz,
                consenususfeature_use_subelements
                )



if __name__ == "__main__":
    main()





    # This goes into the class
    def _init_0(self):
        self.inst = shared_ptr[_String](new _String())
    
    def __init__(self, *args):
        if not args:
             self._init_0(*args)
        elif (len(args)==1):
            self.inst = convString(args[0])
        else:
             raise Exception('can not handle type of %s' % (args,)) 

    def toString(self):
        """Cython signature: str toString()
        -- Note: this returns a unicode string and assumes the input is UTF8 encoded.
        """
        # Decodes the C string to unicode
        cdef char* c_string = _cast_const_away(self.inst.get().c_str())
        cdef Py_ssize_t length = self.inst.get().length()
        ustring = c_string[:length].decode('UTF-8')
        return ustring

    def __bytes__(self):
        return self.c_str()

    def __str__(self):
        # In Python 2 this is supposed to return bytes, in Python 3 it is
        # supposed to return Unicode. We cannot really support both, so we
        # return bytes (as a str object for Python 3).
        # Dont use this in serious code.
        return str(self.c_str())

    def __unicode__(self):
        return self.toString()

    def __repr__(self):
        return self.c_str()

    def c_str(self):
        """Cython signature: const_char * c_str()"""
        # See https://cython.readthedocs.io/en/latest/src/tutorial/strings.html
        #    py_string = <bytes> c_string
        # This creates a Python byte string object that holds a copy of the
        # original C string. It can be safely passed around in Python code, and
        # will be garbage collected when the last reference to it goes out of
        # scope. It is important to remember that null bytes in the string act
        # as terminator character, as generally known from C. The above will
        # therefore only work correctly for C strings that do not contain null
        # bytes.
        cdef const_char  * _r = _cast_const_away(self.inst.get().c_str())
        cdef Py_ssize_t length = self.inst.get().length()
        py_result = _r[:length] # This will work correctly also if the char array contains null bytes
        return py_result




from Types cimport *
from libcpp cimport bool
from Weights cimport *
from MassDecomposer cimport *

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IntegerMassDecomposer.h>" namespace "OpenMS::ims":
    
    cdef cppclass IntegerMassDecomposer[ValueType,DecompositionValueType]:
        # wrap-ignore
        # no-pxd-import

        # We could get this to work but only with int,int and not with long
        # unsigned int,unsigned int and then we may run into all kinds of
        # problems.

        # wrap-instances:
        #   IntegerMassDecomposer := IntegerMassDecomposer[int, int]
        IntegerMassDecomposer() nogil except + # wrap-ignore
        IntegerMassDecomposer(IntegerMassDecomposer) nogil except + #wrap-ignore
        IntegerMassDecomposer(IMSWeights & alphabet) nogil except +
        bool exist(ValueType mass) nogil except +

        # Works in theory but may produce duplicate 
        # libcpp_vector[libcpp_vector[int]] getDecomposition(ValueType mass) nogil except +
        # DecompositionValueType getNumberOfDecompositions(ValueType mass) nogil except +

        # Does not work
        # libcpp_vector[libcpp_vector[int]] getAllDecompositions(ValueType mass) nogil except +





from Types cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from FeatureMap cimport *
from SwathMap cimport *
from TargetedExperiment cimport *
from LightTargetedExperiment cimport *
from TransformationDescription cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *
from SpectrumAccessOpenMS cimport *
from ISpectrumAccess cimport *
from MRMTransitionGroup cimport *

from SpectrumAccessOpenMS cimport *
from SpectrumAccessOpenMSCached cimport *
###  TODO: enable once we have these classes
### # from SpectrumAccessOpenMSInMemory cimport *
### # from SpectrumAccessQuadMZTransforming cimport *

# typedef OpenSwath::LightTransition TransitionType;
# typedef std::map<String, MRMTransitionGroupType> TransitionGroupMapType;

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMFeatureFinderScoring.h>" namespace "OpenMS":

    cdef cppclass MRMFeatureFinderScoring(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        MRMFeatureFinderScoring() nogil except +

        void pickExperiment(MSExperiment & chromatograms,
                            FeatureMap & output,
                            TargetedExperiment & transition_exp_,
                            TransformationDescription trafo,
                            MSExperiment & swath_map) nogil except +

        ## void pickExperiment(shared_ptr[ SpectrumAccessOpenMS ] input_chrom,
        ##                     FeatureMap& output,
        ##                     LightTargetedExperiment& transition_exp,
        ##                     TransformationDescription trafo,
        ##                     SwathMap swath_map,
        ##                     libcpp_map[String, MRMTransitionGroup[MSChromatogram, LightTransition] ] & transition_group_map)

        void setStrictFlag(bool flag) nogil except +

        # void setMS1Map( shared_ptr[ ISpectrumAccess ] ms1_map) nogil except +
        void setMS1Map( shared_ptr[ SpectrumAccessOpenMS ] ms1_map) nogil except +
        void setMS1Map( shared_ptr[ SpectrumAccessOpenMSCached ] ms1_map) nogil except +
        ### # void setMS1Map( shared_ptr[ SpectrumAccessOpenMSInMemory ] ms1_map) nogil except +
        ### # void setMS1Map( shared_ptr[ SpectrumAccessQuadMZTransforming ] ms1_map) nogil except +

        void scorePeakgroups(MRMTransitionGroup[MSChromatogram, LightTransition] transition_group,
                             TransformationDescription trafo,
                             libcpp_vector[ SwathMap ] swath_maps,
                             FeatureMap& output,
                             bool ms1only) nogil except +

        void prepareProteinPeptideMaps_(LightTargetedExperiment& transition_exp) nogil except +

        # # void mapExperimentToTransitionList(OpenSwath::SpectrumAccessPtr input, LightTargetedExperiment transition_exp,
        # #                                    TransitionGroupMapType& transition_group_map,
        # #                                    TransformationDescription trafo, double rt_extraction_window);




from Types cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/OpenSwathWorkflow.h>" namespace "OpenMS":
    
    cdef cppclass OSW_ChromExtractParams "OpenMS::ChromExtractParams":

        OSW_ChromExtractParams(OSW_ChromExtractParams) nogil except + #wrap-ignore

        double min_upper_edge_dist
        double mz_extraction_window
        bool ppm
        libcpp_string extraction_function
        double rt_extraction_window
        double extra_rt_extract
        double im_extraction_window




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from StringList cimport *

cdef extern from "<OpenMS/FORMAT/TextFile.h>" namespace "OpenMS":
    
    cdef cppclass TextFile "OpenMS::TextFile":
        TextFile() nogil except +
        TextFile(TextFile) nogil except + #wrap-ignore
        TextFile(const String &filename, bool trim_linesalse, Int first_n1) nogil except +
        void load(const String &filename, bool trim_linesalse, Int first_n1) nogil except +
        void store(const String &filename) nogil except +
        void addLine(const String line) nogil except +



from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from PeptideIdentification cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/DeNovoPostScoring.h>" namespace "OpenMS":
    
    cdef cppclass DeNovoPostScoring "OpenMS::DeNovoPostScoring":
        # wrap-ignore
        # ABSTRACT CLASS
        # no-pxd-import
        DeNovoPostScoring() nogil except +
        DeNovoPostScoring(DeNovoPostScoring) nogil except +
        void apply(libcpp_vector[ PeptideIdentification ] &identifications, MSExperiment &exp) nogil except +
        void apply(PeptideIdentification &identification, MSSpectrum &spec) nogil except +




from Types cimport *

cdef extern from "<OpenMS/MATH/MISC/RANSACModelLinear.h>" namespace "OpenMS::Math":

    cdef cppclass RansacModelLinear:
       RansacModelLinear() nogil except +
       RansacModelLinear(RansacModelLinear &) nogil except + # wrap-ignore
       # Other functions use iterators -> cannot be wrapped directly




from Types cimport *
from IMSAlphabetParser cimport *

# cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSAlphabetTextParser.h>" namespace "OpenMS::ims":
#     
#     cdef cppclass IMSAlphabetTextParser(IMSAlphabetParser) :
#         # wrap-inherits:
#         #  IMSAlphabetParser
#         IMSAlphabetTextParser() nogil except + 
#         IMSAlphabetTextParser(IMSAlphabetTextParser) nogil except + #wrap-ignore

#         # ContainerType  getElements() nogil except +
#         # NAMESPACE # void parse(std::istream & is_) nogil except +
# 



from libcpp cimport bool
from Types cimport *
from MSSpectrum cimport *

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/Deisotoper.h>" namespace "OpenMS":
    cdef cppclass Deisotoper:
        Deisotoper() nogil except +
        Deisotoper(Deisotoper) nogil except + # wrap-ignore

# COMMENT: wrap static methods
cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/Deisotoper.h>" namespace "OpenMS::Deisotoper":
        void deisotopeAndSingleCharge(MSSpectrum & spectra,
                double fragment_tolerance,
                bool fragment_unit_ppm,
                int min_charge,
                int max_charge,
                bool keep_only_deisotoped,
                unsigned int min_isopeaks,
                unsigned int max_isopeaks,
                bool make_single_charged,
                bool annotate_charge,
                bool annotate_iso_peak_count,
                bool use_decreasing_model,
                unsigned int start_intensity_check,
                bool add_up_intensity) nogil except + # wrap-attach:Deisotoper
       
    
        void deisotopeAndSingleCharge(MSSpectrum & spectra,
                double fragment_tolerance, 
                bool fragment_unit_ppm) nogil except +  # wrap-attach:Deisotoper wrap-as:deisotopeAndSingleChargeDefault




from Types cimport *
from XMLFile cimport *
from ProteinIdentification cimport *
from PeptideIdentification cimport *
from ResidueModification cimport *
from ModificationDefinitionsSet cimport *

cdef extern from "<OpenMS/FORMAT/OMSSAXMLFile.h>" namespace "OpenMS":
    
    cdef cppclass OMSSAXMLFile(XMLFile) :
        # wrap-inherits:
        #  XMLFile

        OMSSAXMLFile() nogil except +
        OMSSAXMLFile(OMSSAXMLFile) nogil except + #wrap-ignore

        void load(const String & filename,
                  ProteinIdentification & protein_identification,
                  libcpp_vector[ PeptideIdentification ] & id_data,
                  bool load_proteins,
                  bool load_empty_hits) nogil except +

        void setModificationDefinitionsSet(ModificationDefinitionsSet rhs) nogil except +





from Types cimport *
from BaseFeature cimport *
from MSExperiment cimport *
from MSSpectrum cimport*
from SiriusMSFile cimport *
from FeatureMapping cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.pair cimport pair as libcpp_pair



cdef extern from "<OpenMS/ANALYSIS/TARGETED/MetaboTargetedAssay.h>" namespace "OpenMS":

    cdef cppclass MetaboTargetedAssay "OpenMS::MetaboTargetedAssay":

       MetaboTargetedAssay() nogil except +
       MetaboTargetedAssay(MetaboTargetedAssay) nogil except + #wrap-ignore


       libcpp_vector[ MetaboTargetedAssay ] extractMetaboTargetedAssay(MSExperiment& spectra,
                                                                       FeatureMapping_FeatureToMs2Indices& feature_ms2_index,
                                                                       double& precursor_rt_tol,
                                                                       double& precursor_mz_distance,
                                                                       double& cosine_sim_threshold,
                                                                       double& transition_threshold,
                                                                       double& min_fragment_mz,
                                                                       double& max_fragment_mz,
                                                                       bool& method_consensus_spectrum,
                                                                       bool& exclude_ms2_precursor,
                                                                       unsigned int& file_counter) nogil except +

       libcpp_vector[ MetaboTargetedAssay ] extractMetaboTargetedAssayFragmentAnnotation(libcpp_vector[ MetaboTargetedAssay_CompoundSpectrumPair ]& v_cmp_spec,
                                                                                         double& transition_threshold,
                                                                                         double& min_fragment_mz,
                                                                                         double& max_fragment_mz,
                                                                                         bool& use_exact_mass,
                                                                                         bool& exclude_ms2_precursor,
                                                                                         unsigned int& file_counter) nogil except +

    cdef cppclass MetaboTargetedAssay_CompoundSpectrumPair "OpenMS::MetaboTargetedAssay::CompoundSpectrumPair":

        MetaboTargetedAssay_CompoundSpectrumPair() nogil except +
        MetaboTargetedAssay_CompoundSpectrumPair(MetaboTargetedAssay_CompoundSpectrumPair) nogil except + #wrap-ignore



from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerHiRes.h>" namespace "OpenMS":

    cdef cppclass PeakPickerHiRes(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        PeakPickerHiRes()                  nogil except +
        PeakPickerHiRes(PeakPickerHiRes)   nogil except + #wrap-ignore

        void pick(MSSpectrum & input,
                  MSSpectrum & output
                 ) nogil except +

        void pickExperiment(MSExperiment & input,
                            MSExperiment & output
                           ) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerHiRes.h>" namespace "OpenMS::PeakPickerHiRes":
    
    cdef cppclass PeakBoundary "OpenMS::PeakPickerHiRes::PeakBoundary":
        PeakBoundary() nogil except +
        PeakBoundary(PeakBoundary) nogil except + #wrap-ignore
        double mz_min
        double mz_max



from Types cimport *
from MzTab cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/MzTabFile.h>" namespace "OpenMS":

    cdef cppclass MzTabFile:

        MzTabFile() nogil except +

        void store(String filename, MzTab & mz_tab) nogil except +
        void load(String filename, MzTab & mz_tab) nogil except +

        # Does not exist
        # void storeProteinReliabilityColumn(bool store) nogil except +
        # void storePeptideReliabilityColumn(bool store) nogil except +
        # void storePSMReliabilityColumn(bool store) nogil except +
        # void storeSmallMoleculeReliabilityColumn(bool store) nogil except +
        # void storeProteinUriColumn(bool store) nogil except +
        # void storePeptideUriColumn(bool store) nogil except +
        # void storePSMUriColumn(bool store) nogil except +
        # void storeSmallMoleculeUriColumn(bool store) nogil except +
        # void storeProteinGoTerms(bool store) nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from SemanticValidator cimport *
from ControlledVocabulary cimport *

cdef extern from "<OpenMS/FORMAT/VALIDATORS/MzMLValidator.h>" namespace "OpenMS::Internal":
    
    cdef cppclass Internal_MzMLValidator "OpenMS::Internal::MzMLValidator":
        # Internal_MzMLValidator() nogil except +
        Internal_MzMLValidator(Internal_MzMLValidator) nogil except + #wrap-ignore

        Internal_MzMLValidator(CVMappings &mapping, ControlledVocabulary &cv) nogil except +
        # ~MzMLValidator() nogil except +







    def __getitem__(self,  in_0 ):
        assert isinstance(in_0, (int, long)), 'arg in_0 wrong type'
    
        cdef long _idx = (<int>in_0)
        if _idx >= self.inst.get().size():
            raise IndexError("invalid index %d" % _idx)

        cdef _String _r = deref(self.inst.get())[(<int>in_0)]
        py_result = _cast_const_away(<char*>_r.c_str())
        return py_result

    def __setitem__(self, key, value):
        assert isinstance(key, (int, long)), 'arg key wrong type'
        assert (isinstance(value, str) or isinstance(value, unicode) or isinstance(value, bytes) or isinstance(value, String)), 'arg value wrong type'

        cdef long _idx = (<int>key)
        if _idx >= self.inst.get().size():
            raise IndexError("invalid index %d" % _idx)

        cdef shared_ptr[_String] _s = convString(value)
        deref(self.inst.get())[(<int>key)] = deref(_s.get())




# This goes into the PXD file
from DPosition cimport DPosition1 as _DPosition1
from DPosition cimport DPosition2 as _DPosition2
cdef class DPosition1:
    cdef shared_ptr[_DPosition1] inst
cdef class DPosition2:
    cdef shared_ptr[_DPosition2] inst


cdef class DPosition1:

    def __dealloc__(self):
        self.inst.reset()

    def __init__(self, *args , **kwargs):
        if not args:
             self._init_0(*args)
        elif (len(args)==1):
             self._init_1(*args)
        else:
             raise Exception('can not handle type of %s' % (args,))
 
    def _init_0(self):
        self.inst = shared_ptr[_DPosition1](new _DPosition1())

    def _init_1(self, double a):
        self.inst = shared_ptr[_DPosition1](new _DPosition1(a))

    def __getitem__(self, ix):
        if ix != 0:
            raise IndexError("invalid index %d" % ix)
        return deref(self.inst.get())[0]


cdef class DPosition2:

    def __dealloc__(self):
        self.inst.reset()


    def __init__(self, *args , **kwargs):
        if not args:
             self._init_0(*args)
        elif (len(args)==2):
             self._init_1(*args)
        else:
             raise Exception('can not handle type of %s' % (args,))
 
    def _init_0(self):
        self.inst = shared_ptr[_DPosition2](new _DPosition2())

    def _init_1(self, double a, double b):
        self.inst = shared_ptr[_DPosition2](new _DPosition2(a, b))

    def __getitem__(self, ix):
        if ix != 0 and ix != 1:
            raise IndexError("invalid index %d" % ix)
        return deref(self.inst.get())[ix]




from PeptideIdentification cimport *
from DefaultParamHandler cimport *
from Map cimport *
# from ZhangSimilarityScore cimport *
# from MassDecomposition cimport *
# from MassDecompositionAlgorithm cimport *
# from CompNovoIonScoringBase cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/CompNovoIonScoringCID.h>" namespace "OpenMS":
    
    cdef cppclass CompNovoIonScoringCID "OpenMS::CompNovoIonScoringCID":
        CompNovoIonScoringCID() nogil except +
        CompNovoIonScoringCID(CompNovoIonScoringCID) nogil except +
        # TODO OpenMS Map type
        # void scoreSpectrum(Map[ double, IonScore ] & CID_ion_scores, 
        #   MSSpectrum &CID_spec, double precursor_weight, Size charge) nogil except +




import sys
import os.path
import pyopenms as pms
import pprint

def addDataProcessing(obj, params, action):
    if isinstance(obj, pms.MSExperiment):
        result = pms.MSExperiment()
        for spec in obj:
            spec = _addDataProcessing(spec, params, action)
            result.addSpectrum(spec)
    else:
        result = _addDataProcessing(obj, params, action)
    return result

def _addDataProcessing(item, params, action):
    dp = item.getDataProcessing()
    p = pms.DataProcessing()
    p.setProcessingActions(set([action]))
    sw = p.getSoftware()
    sw.setName(os.path.basename(sys.argv[0]))
    sw.setVersion(pms.VersionInfo.getVersion())
    p.setSoftware(sw)
    p.setCompletionTime(pms.DateTime.now())

    for k, v in params.asDict().items():
        p.setMetaValue("parameter: "+k, v)

    dp.append(p)
    item.setDataProcessing(dp)
    return item

def writeParamsIfRequested(args, params):
    if args.write_dict_ini or args.write_ini:
        if args.write_dict_ini:
            with open(args.write_dict_ini, "w") as fp:
                pprint.pprint(params.asDict(), stream=fp)
        if args.write_ini:
            fh = pms.ParamXMLFile()
            fh.store(args.write_ini, params)
        return True
    return False

def updateDefaults(args, defaults):
    if args.ini:
        param = pms.Param()
        fh = pms.ParamXMLFile()
        fh.load(args.ini, param)
        defaults.update(param)
    elif args.dict_ini:
        with open(args.dict_ini, "r") as fp:
            try:
                dd = eval(fp.read())
            except:
                raise Exception("could not parse %s" % args.dict_ini)
        defaults.update(dd)



from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.pair cimport pair as libcpp_pair
from DefaultParamHandler cimport *
from ProgressLogger cimport *
from FASTAFile cimport *
from PreprocessedPairSpectra cimport *
from CrossLinkSpectrumMatch cimport *
from MSExperiment cimport *
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/OpenPepXLAlgorithm.h>" namespace "OpenMS":

    cdef cppclass OpenPepXLAlgorithm(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        OpenPepXLAlgorithm() nogil except +
        OpenPepXLAlgorithm(OpenPepXLAlgorithm) nogil except + #wrap-ignore

        OpenPepXLAlgorithm_ExitCodes run(MSExperiment& unprocessed_spectra,
                                         ConsensusMap& cfeatures,
                                         libcpp_vector[ FASTAEntry ]& fasta_db,
                                         libcpp_vector[ ProteinIdentification ]& protein_ids,
                                         libcpp_vector[ PeptideIdentification ]& peptide_ids,
                                         OPXL_PreprocessedPairSpectra& preprocessed_pair_spectra,
                                         libcpp_vector[libcpp_pair[ size_t, size_t ] ]& spectrum_pairs,
                                         libcpp_vector[ libcpp_vector[ CrossLinkSpectrumMatch ] ]& all_top_csms,
                                         MSExperiment& spectra) nogil except +

cdef extern from "<OpenMS/ANALYSIS/XLMS/OpenPepXLAlgorithm.h>" namespace "OpenMS::OpenPepXLAlgorithm":
    cdef enum OpenPepXLAlgorithm_ExitCodes "OpenMS::OpenPepXLAlgorithm::ExitCodes":
        #wrap-attach:
        #    OpenPepXLAlgorithm
        EXECUTION_OK
        ILLEGAL_PARAMETERS
        UNEXPECTED_RESULT
        INCOMPATIBLE_INPUT_DATA



from Param cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMFeaturePicker.h>" namespace "OpenMS":

    cdef cppclass MRMFeaturePicker:

        MRMFeaturePicker() nogil except +
        MRMFeaturePicker(MRMFeaturePicker &) nogil except +

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMFeaturePicker.h>" namespace "OpenMS::MRMFeaturePicker":

    cdef cppclass MRMFP_ComponentParams "OpenMS::MRMFeaturePicker::ComponentParams":

        MRMFP_ComponentParams() nogil except +
        MRMFP_ComponentParams(MRMFP_ComponentParams &) nogil except +

        String component_name
        String component_group_name
        Param params

    cdef cppclass MRMFP_ComponentGroupParams "OpenMS::MRMFeaturePicker::ComponentGroupParams":

        MRMFP_ComponentGroupParams() nogil except +
        MRMFP_ComponentGroupParams(MRMFP_ComponentGroupParams &) nogil except +

        String component_group_name
        Param params



from Types cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/PeptideProteinResolution.h>" namespace "OpenMS":
    
    cdef cppclass PeptideProteinResolution "OpenMS::PeptideProteinResolution":
        PeptideProteinResolution(PeptideProteinResolution) nogil except + #wrap-ignore
        PeptideProteinResolution(bool statistics) nogil except +

        void buildGraph(ProteinIdentification & protein, 
                        libcpp_vector[ PeptideIdentification ] & peptides) nogil except +

        void resolveGraph(ProteinIdentification & protein,
                          libcpp_vector[ PeptideIdentification ] & peptides) nogil except +

        PeptideProteinResolution_ConnectedComponent findConnectedComponent(Size & root_prot_grp) nogil except +

        void resolveConnectedComponent(PeptideProteinResolution_ConnectedComponent & conn_comp,
                                       ProteinIdentification & protein,
                                       libcpp_vector[ PeptideIdentification ] &
                                       peptides) nogil except +


cdef extern from "<OpenMS/ANALYSIS/ID/PeptideProteinResolution.h>" namespace "OpenMS":
    
    cdef cppclass PeptideProteinResolution_ConnectedComponent "OpenMS::ConnectedComponent":
        PeptideProteinResolution_ConnectedComponent() nogil except +
        PeptideProteinResolution_ConnectedComponent(PeptideProteinResolution_ConnectedComponent) nogil except + #wrap-ignore
        libcpp_set[ size_t ] prot_grp_indices
        libcpp_set[ size_t ] pep_indices



from Types cimport *
from ProgressLogger cimport *
from String cimport *
from SVMWrapper cimport *

cdef extern from "<OpenMS/FORMAT/LibSVMEncoder.h>" namespace "OpenMS":
    
    cdef cppclass LibSVMEncoder "OpenMS::LibSVMEncoder":
        LibSVMEncoder() nogil except +
        LibSVMEncoder(LibSVMEncoder) nogil except + #wrap-ignore


# COMMENT: wrap static methods
cdef extern from "<OpenMS/FORMAT/LibSVMEncoder.h>" namespace "OpenMS::LibSVMEncoder":
        
        # static members
        libcpp_vector[double] predictPeptideRT(libcpp_vector[String] sequences,
                                               const SVMWrapper & svm,
                                               const String & allowed_characters,
                                               UInt maximum_sequence_length) nogil except + # wrap-attach:LibSVMEncoder



from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from Compomer cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/ChargePair.h>" namespace "OpenMS":

    cdef cppclass ChargePair:
  
        ChargePair() nogil except +
        ChargePair(ChargePair) nogil except + 
  
        ChargePair(Size index0,
               Size index1,
               Int charge0,
               Int charge1,
               Compomer compomer,
               double mass_diff,
                   bool active) nogil except +

        Int getCharge(UInt pairID) nogil except +
        void setCharge(UInt pairID, Int e) nogil except +
    
        Size getElementIndex(UInt pairID) nogil except +
        void setElementIndex(UInt pairID, Size e) nogil except +
    
        Compomer getCompomer() nogil except +
        void setCompomer( Compomer & compomer) nogil except +
    
        double getMassDiff() nogil except +
        void setMassDiff(double mass_diff) nogil except +
    
        double getEdgeScore() nogil except +
        void setEdgeScore(double score) nogil except +
    
        bool isActive() nogil except +
        void setActive( bool active) nogil except +




from Types cimport *

from PeakFileOptions cimport *
from MzMLFile cimport *
from OnDiscMSExperiment cimport *

cdef extern from "<OpenMS/FORMAT/IndexedMzMLFileLoader.h>" namespace "OpenMS":

    cdef cppclass IndexedMzMLFileLoader:

        IndexedMzMLFileLoader() nogil except +
 
        bool load(String, OnDiscMSExperiment &) nogil except+
        void store(String, OnDiscMSExperiment &) nogil except+
        void store(String, MSExperiment &) nogil except+

        PeakFileOptions getOptions() nogil except +
        void setOptions(PeakFileOptions) nogil except +




from libcpp.string cimport string as libcpp_string
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from String cimport *
# see OpenMS/DATASTRUCTURES/ListUtils.h
ctypedef libcpp_vector[ String ] StringList




from Types cimport *
from FeatureMap cimport *
from MRMFeatureQC cimport *
from DefaultParamHandler cimport *
from TargetedExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMFeatureFilter.h>" namespace "OpenMS":

    cdef cppclass MRMFeatureFilter(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler

        MRMFeatureFilter() nogil except +
        MRMFeatureFilter(MRMFeatureFilter &) nogil except +

        void FilterFeatureMap(FeatureMap features, MRMFeatureQC filter_criteria, TargetedExperiment transitions) nogil except +

        # could add support for other members if needed
        # however, they are only used internally for now



from Types cimport *
from String cimport *
from Element cimport *
from IsotopeDistribution cimport *

cdef extern from "<OpenMS/CHEMISTRY/EmpiricalFormula.h>" namespace "OpenMS":

    cdef cppclass EmpiricalFormula:

        EmpiricalFormula() nogil except +
        EmpiricalFormula(EmpiricalFormula) nogil except + # wrap-ignore

        # constructor from string
        EmpiricalFormula(String) nogil except +

        # constructor with element pointer and number
        EmpiricalFormula(SignedSize number, Element * element, SignedSize charge) nogil except +

        # returns the mono isotopic weight of the formula (includes proton charges)
        double getMonoWeight() nogil except +

        # returns the average weight of the formula (includes proton charges)
        double getAverageWeight() nogil except +

        # Fills this EmpiricalFormula with an approximate elemental composition for a given average weight and approximate elemental stoichiometry
        bool estimateFromWeightAndComp(double average_weight, double C, double H, double N, double O, double S, double P) nogil except +

        # Fills this EmpiricalFormula with an approximate elemental composition for a given average weight,
        # exact number of sulfurs, and approximate elemental stoichiometry
        bool estimateFromWeightAndCompAndS(double average_weight, UInt S, double C, double H, double N, double O, double P) nogil except +

        # Computes the isotope distribution of an empirical formula using the CoarseIsotopePatternGenerator or the FineIsotopePatternGenerator method
        IsotopeDistribution getIsotopeDistribution(CoarseIsotopePatternGenerator) nogil except +
        IsotopeDistribution getIsotopeDistribution(FineIsotopePatternGenerator) nogil except +

        # returns the fragment isotope distribution of this conditioned
        # on a precursor formula and a list of isolated precursor isotopes.
        # precursor: the empirical formula of the precursor
        # precursor_isotopes: the set of precursor isotopes that were isolated
        # method: the method that will be used for the calculation of the IsotopeDistribution
        IsotopeDistribution getConditionalFragmentIsotopeDist(EmpiricalFormula& precursor, libcpp_set[ unsigned int ]& precursor_isotopes, CoarseIsotopePatternGenerator method) nogil except +

        # returns the number of atoms
        # doesnt work!
        ## Size getNumberOf(Element * element) nogil except +

        # returns the atoms total
        Size getNumberOfAtoms() nogil except +

        # returns the charge
        SignedSize getCharge() nogil except +

        # sets the charge
        void setCharge(SignedSize charge) nogil except +

        # returns the formula as a string (charges are not included)
        String toString() nogil except +

        # returns the formula as a hash
        libcpp_map[libcpp_string, int] toMap() nogil except + #wrap-as:getElementalComposition wrap-doc:Get elemental composition as a hash {'Symbol' -> NrAtoms}


        # returns true if the formula does not contain a element
        bool isEmpty() nogil except +

        # returns true if charge != 0
        bool isCharged() nogil except +

        # returns true if the formula contains the element
        bool hasElement(Element * element) nogil except +

        # returns true if all elements from @p ef are LESS abundant (negative allowed) than the corresponding elements of this EmpiricalFormula
        bool contains(EmpiricalFormula ef) nogil except +

        # returns true if the formulas contain equal elements in equal quantities
        bool operator==(EmpiricalFormula) nogil except +

        # returns true if the formulas differ in elements composition
        bool operator!=(EmpiricalFormula) nogil except +

        EmpiricalFormula operator+(EmpiricalFormula) nogil except +
        # EmpiricalFormula operator-(EmpiricalFormula) nogil except +
        # EmpiricalFormula operator*(EmpiricalFormula) nogil except +

        EmpiricalFormula iadd(EmpiricalFormula)   nogil except + # wrap-as:operator+=
        # EmpiricalFormula iminus(EmpiricalFormula)   nogil except + # wrap-as:operator-=

        double calculateTheoreticalIsotopesNumber() nogil except +
        



# part of the SpectraFilters
from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/BernNorm.h>" namespace "OpenMS":

    cdef cppclass BernNorm(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        BernNorm()           nogil except +
        BernNorm(BernNorm)   nogil except + #wrap-ignore

        void filterSpectrum(MSSpectrum & spec) nogil except +
        void filterPeakSpectrum(MSSpectrum & spec) nogil except +
        void filterPeakMap(MSExperiment & exp) nogil except +




from libcpp cimport bool
from Types cimport *
from String cimport *
from Residue cimport *
from EmpiricalFormula cimport *
from ResidueModification cimport *
from Map cimport *

cdef extern from "<OpenMS/CHEMISTRY/AASequence.h>" namespace "OpenMS":

    cdef cppclass AASequence:
        # wrap-hash:
        #   toString().c_str()
        #
        # wrap-doc:
        #   Representation of a peptide/protein sequence
        #   This class represents amino acid sequences in OpenMS. An AASequence
        #   instance primarily contains a sequence of residues. 

        AASequence() nogil except +
        AASequence(AASequence) nogil except + # wrap-ignore

        AASequence operator+(AASequence)    nogil except +
        AASequence iadd(AASequence)   nogil except + # wrap-as:operator+=

        # Note that this is a const-ref, so we cannot easily set residues
        Residue operator[](int) nogil except + # wrap-upper-limit:size()

        # check if sequence is empty
        bool empty() nogil except +

        # returns the peptide as string with modifications embedded in brackets
        String toString() nogil except +

        # returns the peptide as string without any modifications
        String toUnmodifiedString() nogil except +

        # returns the peptide as string without any modifications
        String toUniModString() nogil except +

        String toBracketString() nogil except + #wrap-doc:create a TPP compatible string of the modified sequence using bracket notation. Uses integer mass by default.
        String toBracketString(bool integer_mass) nogil except + #wrap-doc:create a TPP compatible string of the modified sequence using bracket notation.
        String toBracketString(bool integer_mass, bool mass_delta) nogil except + #wrap-doc:create a TPP compatible string of the modified sequence using bracket notation.
        String toBracketString(bool integer_mass, bool mass_delta, libcpp_vector[String] fixed_modifications) nogil except + #wrap-doc:create a TPP compatible string of the modified sequence using bracket notation.

        # set the modification of the residue at position index
        void setModification(Size index, String modification) nogil except +

        # sets the N-terminal modification
        void setNTerminalModification(String modification) nogil except +

        # returns the name (ID) of the N-terminal modification, or an empty string if none is set
        String getNTerminalModificationName() nogil except +

        # sets the C-terminal modification
        void setCTerminalModification(String modification) nogil except +

        const ResidueModification * getNTerminalModification() nogil except +

        # returns the name (ID) of the C-terminal modification, or an empty string if none is set nogil except +
        String getCTerminalModificationName() nogil except +

        const ResidueModification * getCTerminalModification() nogil except +

        # returns the residue at position index
        Residue getResidue(Size index) nogil except +

        # returns the formula of the peptide
        EmpiricalFormula getFormula() nogil except + # wrap-doc:convenience function with ResidueType=Full and charge = 0 by default
        EmpiricalFormula getFormula(ResidueType type_, Int charge) nogil except +

        # returns the average weight of the peptide
        double getAverageWeight() nogil except +
        double getAverageWeight(ResidueType type_, Int charge) nogil except +

        # returns the mono isotopic weight of the peptide
        double getMonoWeight() nogil except +
        double getMonoWeight(ResidueType type_, Int charge) nogil except +

        # returns the number of residues
        Size size() nogil except +

        # returns a peptide sequence of the first index residues
        AASequence getPrefix(Size index) nogil except +

        # returns a peptide sequence of the last index residues
        AASequence getSuffix(Size index) nogil except +

        # returns a peptide sequence of number residues, beginning at position index
        AASequence getSubsequence(Size index, UInt number) nogil except +

        # compute frequency table of amino acids
        void getAAFrequencies(Map[String, size_t]) nogil except + # wrap-ignore

        # returns true if the peptide contains the given residue
        bool has(Residue residue) nogil except +

        # returns true if the peptide contains the given peptide
        # @note c-term and n-term mods are ignored
        bool hasSubsequence(AASequence peptide) nogil except +

        # returns true if the peptide has the given prefix
        # n-term mod is also checked (c-term as well, if prefix is of same length)
        bool hasPrefix(AASequence peptide) nogil except +

        # returns true if the peptide has the given suffix
        # c-term mod is also checked (n-term as well, if suffix is of same length)
        bool hasSuffix(AASequence peptide) nogil except +

        # predicate which is true if the peptide is N-term modified
        bool hasNTerminalModification() nogil except +

        # predicate which is true if the peptide is C-term modified
        bool hasCTerminalModification() nogil except +

        # returns true if any of the residues or termini are modified
        bool isModified() nogil except +

# COMMENT: wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/AASequence.h>" namespace "OpenMS::AASequence":
        
        # static members
        AASequence fromString(String s, bool permissive) nogil except +  # wrap-attach:AASequence wrap-as:fromStringPermissive
        
        # static members
        AASequence fromString(String s) nogil except +  # wrap-attach:AASequence




from Types cimport *
from CsiAdapterIdentification cimport CsiAdapterIdentification

cdef extern from "<OpenMS/FORMAT/DATAACCESS/CsiFingerIdMzTabWriter.h>" namespace "OpenMS::CsiFingerIdMzTabWriter":
    
    cdef cppclass CsiAdapterRun "OpenMS::CsiFingerIdMzTabWriter::CsiAdapterRun":
        CsiAdapterRun() nogil except +
        CsiAdapterRun(CsiAdapterRun) nogil except + #wrap-ignore
        libcpp_vector[ CsiAdapterIdentification ] identifications




from Types cimport *
from libcpp cimport bool
from String cimport *

cdef extern from "<OpenMS/FORMAT/XMLFile.h>" namespace "OpenMS::Internal":
    
    cdef cppclass XMLFile "OpenMS::Internal::XMLFile":
        XMLFile() nogil except +
        XMLFile(XMLFile) nogil except + #wrap-ignore

        XMLFile(const String & schema_location, const String & version) nogil except +
        # NAMESPACE # bool isValid(const String & filename, std::ostream & os) nogil except +
        String getVersion() nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from Adduct cimport *
from Compomer cimport *

ctypedef libcpp_vector[Adduct] AdductsType

cdef extern from "<OpenMS/DATASTRUCTURES/MassExplainer.h>" namespace "OpenMS":
    
    cdef cppclass MassExplainer "OpenMS::MassExplainer":
        MassExplainer() nogil except +
        MassExplainer(MassExplainer) nogil except + #wrap-ignore

        MassExplainer(libcpp_vector[Adduct] adduct_base) nogil except +
        MassExplainer(Int q_min, Int q_max, Int max_span, double thresh_logp) nogil except +
        ## MassExplainer(libcpp_vector[Adduct] adduct_base, Int q_min, Int q_max, Int max_span, double thresh_logp, Size max_neutrals) nogil except +
        ## MassExplainer operator=(MassExplainer &rhs) nogil except +
        void setAdductBase(libcpp_vector[Adduct] adduct_base) nogil except +
        libcpp_vector[Adduct] getAdductBase() nogil except +
        Compomer getCompomerById(Size id) nogil except +
        void compute() nogil except +




from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from ProgressLogger cimport *
from TargetedExperiment cimport *
from TransformationDescription cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/ChromatogramExtractor.h>" namespace "OpenMS":

    cdef cppclass ChromatogramExtractor(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger

        ChromatogramExtractor()                  nogil except +
        ChromatogramExtractor(ChromatogramExtractor)   nogil except + 

        void extractChromatograms(MSExperiment & input,
                                  MSExperiment & output, 
                                  TargetedExperiment & transition_exp,
                                  double extract_window,
                                  bool ppm,
                                  TransformationDescription trafo,
                                  double rt_extraction_window,
                                  String filter) nogil except +

        # TODO immutable types by reference
        # void extract_value_tophat(MSSpectrum input, double mz,
        #  Size peak_idx, double integrated_intensity, double extract_window, bool ppm)
        # void extract_value_bartlett(MSSpectrum input, double mz,
        #  Size peak_idx, double integrated_intensity, double extract_window, bool ppm)
    



# part of the SpectraFilters
from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/ThresholdMower.h>" namespace "OpenMS":

    cdef cppclass ThresholdMower(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        ThresholdMower()          nogil except +
        ThresholdMower(ThresholdMower) nogil except + #wrap-ignore

        void filterSpectrum(MSSpectrum & spec) nogil except +
        void filterPeakSpectrum(MSSpectrum & spec) nogil except +
        void filterPeakMap(MSExperiment & exp) nogil except +




from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerMaxima.h>" namespace "OpenMS":

    cdef cppclass PeakPickerMaxima:

        PeakPickerMaxima() nogil except + # wrap-ignore
        PeakPickerMaxima(PeakPickerMaxima &) nogil except +
        PeakPickerMaxima(double signal_to_noise, double spacing_difference, double sn_window_length) nogil except +

        void findMaxima(libcpp_vector[double] mz_array, libcpp_vector[double]
                        int_array, libcpp_vector[PeakCandidate]& pc) nogil except +

        void pick(libcpp_vector[double] mz_array, libcpp_vector[double]
                        int_array, libcpp_vector[PeakCandidate]& pc) nogil except +
          
cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerMaxima.h>" namespace "OpenMS::PeakPickerMaxima":
    
    cdef cppclass PeakCandidate "OpenMS::PeakPickerMaxima::PeakCandidate":

        PeakCandidate() nogil except +
        PeakCandidate(PeakCandidate &) nogil except +
        int pos
        int left_boundary
        int right_boundary
        double mz_max
        double int_max



from String cimport *
from Param cimport *
from StringList cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/SYSTEM/File.h>" namespace "OpenMS":

    cdef cppclass File:
        pass

# File has only methods, which we wrap as declared below:
cdef extern from "<OpenMS/SYSTEM/File.h>" namespace "OpenMS::File":

    String getExecutablePath()  # wrap-attach:File

    # Method used to test if a @p file exists.
    bool exists(String file_) nogil except + # wrap-attach:File

    # Return true if the file does not exist or the file is empty
    bool empty(String file_) nogil except + # wrap-attach:File

    # Removes a file (if it exists).
    bool remove(String file_) nogil except + # wrap-attach:File

    # Removes the specified directory (absolute path). Returns true if successful.
    bool removeDirRecursively(String dir_name) nogil except + # wrap-attach:File

    # Replaces the relative path in the argument with the absolute path.
    String absolutePath(String file) nogil except + # wrap-attach:File

    # Returns the basename of the file (without the path).
    String basename(String file) nogil except + # wrap-attach:File

    # Returns the path of the file (without the file name).
    String path(String file) nogil except + # wrap-attach:File

    # Return true if the file exists and is readable
    bool readable(String file) nogil except + # wrap-attach:File

    # Return true if the file is writable
    bool writable(String file) nogil except + # wrap-attach:File

    # Return true if the given path specifies a directory
    bool isDirectory(String path) nogil except + # wrap-attach:File

    # @brief Looks up the location of the file @p filename
    String find(String filename, StringList directories) nogil except + # wrap-attach:File

    # @brief Retrieves a list of files matching @p file_pattern in directory
    #        @p dir (returns filenames without paths unless @p full_path is true)
    bool fileList(String dir, String file_pattern, StringList output, bool full_path) nogil except + # wrap-attach:File

    # Returns a string, consisting of date, time, hostname, process id, and a incrementing number.  This can be used for temporary files.
    String getUniqueName() nogil except + # wrap-attach:File

    # Returns the OpenMS data path (environment variable overwrites the default installation path)
    String getOpenMSDataPath() nogil except + # wrap-attach:File

    String getOpenMSHomePath() nogil except + # wrap-attach:File

    # The current OpenMS temporary data path (for temporary files)
    String getTempDirectory() nogil except + # wrap-attach:File

    # The current OpenMS user data path (for result files)
    String getUserDirectory() nogil except + # wrap-attach:File

    # get the system's default OpenMS.ini file in the users home directory (&lt nogil except +home&gt nogil except +/OpenMS/OpenMS.ini)
    # or create/repair it if required
    Param getSystemParameters() nogil except + # wrap-attach:File

    # uses File::find() to search for a file names @p db_name
    # in the 'id_db_dir' param of the OpenMS system parameters
    # @exception FileNotFound is thrown, if the file is not found
    String findDatabase(String db_name) nogil except + # wrap-attach:File

    # Searchs for an executable with the given name.
    String findExecutable(String toolName) nogil except + # wrap-attach:File

    String getTemporaryFile(const String & alternative_file) nogil except + # wrap-attach:File

    # Resolves a partial file name to a documentation file in the doc-folder.
    String findDoc(String filename) nogil except + # wrap-attach:File

    bool rename(const String & from_, const String & to, bool overwrite_existing, bool verbose) nogil except + # wrap-attach:File

    # bool removeDir(const QString & dir_name) nogil except +




from UniqueIdInterface cimport setUniqueId as _setUniqueId


    def setUniqueIds(self):
        self.inst.get().applyMemberFunction(address(_setUniqueId))

    def getColumnHeaders(self):

        # ColumnHeaders is a type alias for Map<..> which can not be
        # handled by autowrap automatically. So we have to provide a manual
        # converter here:
        #
        # (the wrapper works on linux, but msvc complains
        # a lot about the generated code... uwe schmitt)

        cdef ColumnHeaders _r = self.inst.get().getColumnHeaders()
        py_result = dict()
        cdef ColumnHeaders_iterator it__r = _r.begin()
        cdef ColumnHeader item_py_result
        while it__r != _r.end():
           item_py_result = ColumnHeader.__new__(ColumnHeader)
           item_py_result.inst = shared_ptr[_ColumnHeader](new _ColumnHeader((deref(it__r)).second))
           py_result[<unsigned long int>(deref(it__r).first)] = item_py_result
           inc(it__r)
        return py_result

    def setColumnHeaders(self, dict in_0 ):
        assert isinstance(in_0, dict) and all(isinstance(k, (int, long)) for k in in_0.keys()) and all(isinstance(v, ColumnHeader) for v in in_0.values()), 'arg in_0 wrong type'
        cdef ColumnHeaders v0
        for key, value in in_0.items():
           v0[<unsigned long int> key] = deref((<ColumnHeader>value).inst.get())

        # we have to utilize AutowrapRefHolder here, because cython does not
        # like
        #
        #     self.inst.get().getColumnHeaders() = v0
        #
        # and if you choose
        #
        #     cdef ColumnHeaders & ref = self.inst.get().getColumnHeaders()
        #     ref = v0
        #
        # the c++ compiler will complain, as cython generates invalid code in
        # this case: Cython creates two statements from the first line:
        #
        #     ColumnHeader & ref;   // invalid: ref not initailized
        #     ref = self.inst.get().getColumnHeaders()
        #
        # wrapping this ref into a class holding the ref and using pointers
        # works. the following code results in c++ code simialar to
        #
        #    AutowrapRefHolder<ColumnHeader> * refholder;
        #    refholder = new AutowrapRefHolder<ColumnHeader>(self.inst...)
        #    refholder.assign(v0)

        cdef AutowrapRefHolder[ColumnHeaders] * refholder
        refholder = new AutowrapRefHolder[ColumnHeaders](self.inst.get().getColumnHeaders())
        refholder.assign(v0)

        cdef replace_in_0 = dict()
        cdef ColumnHeaders_iterator it_in_0 = v0.begin()
        cdef ColumnHeader item_in_0
        while it_in_0 != v0.end():
           item_in_0 = ColumnHeader.__new__(ColumnHeader)
           item_in_0.inst = shared_ptr[_ColumnHeader](new _ColumnHeader((deref(it_in_0)).second))
           replace_in_0[<unsigned long int> deref(it_in_0).first] = item_in_0
           inc(it_in_0)
        in_0.clear()
        in_0.update(replace_in_0)



from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set
from libcpp cimport bool
from Types cimport *
from String cimport *
from EmpiricalFormula cimport *

cdef extern from "<OpenMS/CHEMISTRY/DigestionEnzymeProtein.h>" namespace "OpenMS":

    cdef cppclass DigestionEnzymeProtein:
        DigestionEnzymeProtein() nogil except + # wrap-ignore

        DigestionEnzymeProtein(DigestionEnzymeProtein) nogil except + # wrap-ignore

        # detailed constructor
        DigestionEnzymeProtein(String name,
                               String cleavage_regex,
                               libcpp_set[String] synonyms,
                               String regex_description,
                               EmpiricalFormula n_term_gain,
                               EmpiricalFormula c_term_gain,
                               String psi_id,
                               String xtandem_id,
                               UInt comet_id,
                               UInt omssa_id) nogil except +

        # sets the name of the Enzyme
        void setName(String name) nogil except +

        # returns the name of the Enzyme
        String getName() nogil except +

        # sets the synonyms
        void setSynonyms(libcpp_set[String] synonyms) nogil except +

        # adds a synonym
        void addSynonym(String synonym) nogil except +

        # returns the sysnonyms
        libcpp_set[String] getSynonyms() nogil except +

        # sets the name of the Enzyme as three letter code
        void setRegEx(String three_letter_code) nogil except +

        # returns the name of the Enzyme as three letter code
        String getRegEx() nogil except +

        # sets the regex description
        void setRegExDescription(String one_letter_code) nogil except +

        # returns the regex description
        String getRegExDescription() nogil except +

        # sets the N-term gain
        void setNTermGain(EmpiricalFormula value) nogil except +

        # sets the C-term gain
        void setCTermGain(EmpiricalFormula value) nogil except +

        # returns the N-term gain
        EmpiricalFormula getNTermGain() nogil except +

        # returns the C-term gain
        EmpiricalFormula getCTermGain() nogil except +

        # sets the PSI ID
        void setPSIID(String value) nogil except +

        # returns the PSI ID
        String getPSIID() nogil except +

        void setXTandemID(String value) nogil except +
        String getXTandemID() nogil except +

        String getCruxID() nogil except +
        void setCruxID(const String & value) nogil except +

        void setCometID(int value) nogil except +
        int getCometID() nogil except +

        # sets the OMSSA ID
        void setOMSSAID(int value) nogil except +
        int getOMSSAID() nogil except +

        void setMSGFID(Int value) nogil except +
        Int getMSGFID() nogil except +

        # equality operator
        bool operator==(DigestionEnzymeProtein& Enzyme) nogil except +

        # inequality operator
        bool operator!=(DigestionEnzymeProtein& Enzyme) nogil except +

        # equality operator for cleavage regex
        bool operator==(EmpiricalFormula cleavage_regex) nogil except +

        # equality operator for cleavage regex
        bool operator!=(EmpiricalFormula cleavage_regex) nogil except +

        bool setValueFromFile(const String & key, const String & value) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from ProteinIdentification cimport *
from PeptideIdentification cimport *

from String cimport *
from ProgressLogger cimport *
from Identification cimport *

cdef extern from "<OpenMS/FORMAT/MzIdentMLFile.h>" namespace "OpenMS":

    cdef cppclass MzIdentMLFile(ProgressLogger):
        # wrap-inherits:
        #   ProgressLogger

        MzIdentMLFile() nogil except +

        void load(String filename, libcpp_vector[ProteinIdentification] & poid, libcpp_vector[PeptideIdentification] & peid) nogil except +
        void store(String filename, libcpp_vector[ProteinIdentification] & poid, libcpp_vector[PeptideIdentification] & peid) nogil except +
        bool isSemanticallyValid(String filename, StringList errors, StringList warnings) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from PeptideIdentification cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/CompNovoIdentification.h>" namespace "OpenMS":

    cdef cppclass CompNovoIdentification(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        CompNovoIdentification()      nogil except +
        CompNovoIdentification(CompNovoIdentification)      nogil except + #private

        void getIdentifications(libcpp_vector[PeptideIdentification] & ids, MSExperiment)      nogil except +
        void getIdentification(PeptideIdentification & id, MSSpectrum cid_spec, MSSpectrum etd_spec)      nogil except +




from String cimport *

from MSSpectrum cimport *
from Peak1D cimport *
from FeatureMap cimport *
from Feature cimport *

cdef extern from "<OpenMS/FORMAT/KroenikFile.h>" namespace "OpenMS":

    cdef cppclass KroenikFile:

        KroenikFile() nogil except +

        void store(String filename, MSSpectrum & spectrum)  nogil except +
        void load(String filename, FeatureMap & feature_map) nogil except +



from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerIterative.h>" namespace "OpenMS":

    cdef cppclass PeakPickerIterative(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        PeakPickerIterative() nogil except +
        PeakPickerIterative(PeakPickerIterative) nogil except + #wrap-ignore

        void pick(MSSpectrum & input,
                  MSSpectrum & output
                 ) nogil except +

        void pickExperiment(MSExperiment & input,
                            MSExperiment & output
                           ) nogil except +



# cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakPickerIterative.h>" namespace "OpenMS":
# 
#     cdef cppclass PeakCandidate:
#         PeakCandidate() nogil except +
#         PeakCandidate(PeakCandidate) nogil except +
#         int index
#         double peak_apex_intensity
# 
#         double integrated_intensity
#         double leftWidth
#         double rightWidth
#         float mz
# 



from Types cimport *
from ConsensusIDAlgorithmSimilarity cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ConsensusIDAlgorithmPEPIons.h>" namespace "OpenMS":
    
    cdef cppclass ConsensusIDAlgorithmPEPIons(ConsensusIDAlgorithmSimilarity) :
        # wrap-inherits:
        #  ConsensusIDAlgorithmSimilarity
        ConsensusIDAlgorithmPEPIons() nogil except +
        ConsensusIDAlgorithmPEPIons(ConsensusIDAlgorithmPEPIons) nogil except + #wrap-ignore




from Types cimport *
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/PeakPickerMRM.h>" namespace "OpenMS":
    
    cdef cppclass PeakPickerMRM(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        PeakPickerMRM() nogil except +
        PeakPickerMRM(PeakPickerMRM) nogil except + #wrap-ignore

        void pickChromatogram(MSChromatogram & chromatogram, MSChromatogram & picked_chrom) nogil except +




import pyopenms

def testCVTermList():

    term = pyopenms.CVTerm()

    term.setAccession("ACC")
    assert term.getAccession() == "ACC"

    term.setName("NAME")
    assert term.getName() == "NAME"

    term.setCVIdentifierRef("CVREF")
    assert term.getCVIdentifierRef() == "CVREF"

    term.setValue(123)
    assert term.getValue() == 123

    li = pyopenms.CVTermList()

    li.setCVTerms([term])

    (name, (term,)), =  li.getCVTerms().items()

    assert name == b"ACC"
    assert term.getName() == "NAME"
    assert term.getCVIdentifierRef() == "CVREF"
    assert term.getValue() == 123

    li.replaceCVTerm(term)
    (name, (term,)), =  li.getCVTerms().items()

    assert name == b"ACC"
    assert term.getName() == "NAME"
    assert term.getCVIdentifierRef() == "CVREF"
    assert term.getValue() == 123

    li.replaceCVTerms([term], "ACC2")

    dd = li.getCVTerms()
    assert len(dd) == 2
    assert b"ACC" in dd and b"ACC2" in dd
    term, = dd[b"ACC"]
    assert term.getName() == "NAME"
    assert term.getCVIdentifierRef() == "CVREF"
    assert term.getValue() == 123
    term, = dd[b"ACC2"]
    assert term.getName() == "NAME"
    assert term.getCVIdentifierRef() == "CVREF"
    assert term.getValue() == 123

    # li.replaceCVTerms(li.getCVTerms())
    # dd =  li.getCVTerms()
    # assert len(dd) == 2
    # assert b"ACC" in dd and b"ACC2" in dd
    # term, = dd[b"ACC"]
    # assert term.getName() == "NAME"
    # assert term.getCVIdentifierRef() == "CVREF"
    # assert term.getValue() == 123
    # term, = dd[b"ACC2"]
    # assert term.getName() == "NAME"
    # assert term.getCVIdentifierRef() == "CVREF"
    # assert term.getValue() == 123

    li.addCVTerm(term)
    dd = li.getCVTerms()

    assert len(dd) == 2
    assert b"ACC" in dd and b"ACC2" in dd

    term1, term2, = dd[b"ACC"]
    assert term1.getName() == "NAME"
    assert term1.getCVIdentifierRef() == "CVREF"
    assert term1.getValue() == 123

    assert term2.getName() == "NAME"
    assert term2.getCVIdentifierRef() == "CVREF"
    assert term2.getValue() == 123

    term, = dd[b"ACC2"]
    assert term.getName() == "NAME"
    assert term.getCVIdentifierRef() == "CVREF"
    assert term.getValue() == 123

    assert li.hasCVTerm("ACC")
    assert not li.hasCVTerm("ABC")




from Types cimport *
from Param cimport *

from TransformationModel cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/AbsoluteQuantitationMethod.h>" namespace "OpenMS":

    cdef cppclass AbsoluteQuantitationMethod:

        AbsoluteQuantitationMethod()  nogil except +
        AbsoluteQuantitationMethod(AbsoluteQuantitationMethod)  nogil except + #wrap-ignore

        void setLLOD(double llod) nogil except +
        void setULOD(double ulod) nogil except +
        double getLLOD() nogil except +
        double getULOD() nogil except +

        void setLLOQ(double lloq) nogil except +
        void setULOQ(double uloq) nogil except +
        double getLLOQ() nogil except +
        double getULOQ() nogil except +

        bool checkLOD(double value) nogil except +
        bool checkLOQ(double value) nogil except +
        
        void setComponentName(const String& component_name) nogil except +
        void setISName(const String& IS_name) nogil except +
        void setFeatureName(const String& feature_name) nogil except +
        String getComponentName() nogil except +
        String getISName() nogil except +
        String getFeatureName() nogil except +

        void setConcentrationUnits(const String& concentration_units) nogil except +
        String getConcentrationUnits() nogil except +

        void setTransformationModel(const String& transformation_model) nogil except +
        void setTransformationModelParams(Param transformation_model_param) nogil except +
        String getTransformationModel() nogil except +
        Param getTransformationModelParams() nogil except +

        void setNPoints(Int n_points) nogil except +
        void setCorrelationCoefficient(double correlation_coefficient) nogil except +
        Int getNPoints() nogil except +
        double getCorrelationCoefficient() nogil except +



from Types cimport *
from String cimport *
from DigestionEnzymeProtein cimport *

cdef extern from "<OpenMS/CHEMISTRY/ProteaseDB.h>" namespace "OpenMS":

    cdef cppclass ProteaseDB "OpenMS::ProteaseDB":
        # wrap-manual-memory:
        #     cdef AutowrapPtrHolder[_ProteaseDB] inst

        ProteaseDB() nogil except + #wrap-ignore
        ProteaseDB(ProteaseDB) nogil except + #wrap-ignore

        const DigestionEnzymeProtein* getEnzyme(const String& name) nogil except +
        const DigestionEnzymeProtein* getEnzymeByRegEx(const String& cleavage_regex) nogil except +
        void getAllNames(libcpp_vector[ String ]& all_names) nogil except +
        void getAllXTandemNames(libcpp_vector[ String ]& all_names) nogil except +
        void getAllOMSSANames(libcpp_vector[ String ]& all_names) nogil except +
        void getAllCometNames(libcpp_vector[ String ]& all_names) nogil except +
        void getAllCruxNames(libcpp_vector[ String ] & all_names) nogil except +
        void getAllMSGFNames(libcpp_vector[ String ] & all_names) nogil except +
        bool hasEnzyme(const String& name) nogil except +
        bool hasRegEx(const String& cleavage_regex) nogil except +
        # bool hasEnzyme(DigestionEnzymeProtein* enzyme) nogil except + # does not make sense as the ptr wont match

        # ConstEnzymeIterator beginEnzyme() nogil except +
        # ConstEnzymeIterator endEnzyme() nogil except +

## wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/ProteaseDB.h>" namespace "OpenMS::ProteaseDB":

    ProteaseDB* getInstance() nogil except + # wrap-ignore



from libcpp.map cimport map as libcpp_map
from libcpp cimport bool
from smart_ptr cimport shared_ptr
from Types cimport *
from ProgressLogger cimport *
from DefaultParamHandler cimport *
from Param cimport *
from SimTypes cimport *
from ConsensusMap cimport *
from MSExperiment cimport *
from FeatureMap cimport *

# SimTypes::MSSimExperiment = MSExperiment
# SimTypes::FeatureMapSim = FeatureMap
cdef extern from "<OpenMS/SIMULATION/MSSim.h>" namespace "OpenMS":

    cdef cppclass MSSim:

        MSSim()      nogil except +
        MSSim(MSSim) nogil except + # wrap-ignore

        # General purpose function to simulate a mass spectrometry run
        #
        #@param rnd_gen random number generator which will be passed to the different classes
        #@param peptides List of peptides and abundances that will be simulated
        void simulate(shared_ptr[SimRandomNumberGenerator] rnd_gen, SampleChannels peptides) nogil except +

        # Access the simulated experiment
        MSExperiment getExperiment() nogil except +

        # Access the simulated features
        FeatureMap getSimulatedFeatures() nogil except +

        # Access the charge consensus map of simulated features
        ConsensusMap getChargeConsensus() nogil except +

        # Access the contaminants feature map of simulated features
        FeatureMap getContaminants() nogil except +

        # Access the labeling consensus map of simulated features
        ConsensusMap getLabelingConsensus() nogil except +

        # Access the labeling consensus map of simulated features
        MSExperiment getPeakMap() nogil except +

        # Returns the default parameters for simulation including the labeling technique with name @p labeling_name
        Param getParameters() nogil except +

        void getIdentifications(libcpp_vector[ ProteinIdentification ] & proteins, libcpp_vector[ PeptideIdentification ] & peptides) nogil except +

        void getMS2Identifications(libcpp_vector[ ProteinIdentification ] & proteins, libcpp_vector[ PeptideIdentification ] & peptides) nogil except +

        void getFeatureIdentifications(libcpp_vector[ ProteinIdentification ] & proteins, libcpp_vector[ PeptideIdentification ] & peptides) nogil except +




from Types cimport *
from ProgressLogger cimport *
from XMLHandler cimport *
from XMLFile cimport *
from String cimport *
from StringList cimport *
from Attachment cimport *

cdef extern from "<OpenMS/FORMAT/QcMLFile.h>" namespace "OpenMS":
    
    cdef cppclass QcMLFile(XMLHandler,XMLFile,ProgressLogger) :
        # wrap-inherits:
        #  XMLHandler
        #  XMLFile
        #  ProgressLogger
        QcMLFile() nogil except +
        QcMLFile(QcMLFile) nogil except + #wrap-ignore
        String map2csv(libcpp_map[ String, libcpp_map[ String, String ] ] & cvs_table, const String & separator) nogil except + # wrap-ignore
        String exportIDstats(const String & filename) nogil except +
        void addRunQualityParameter(String r, QualityParameter qp) nogil except +
        void addRunAttachment(String r, Attachment at) nogil except +
        void addSetQualityParameter(String r, QualityParameter qp) nogil except +
        void addSetAttachment(String r, Attachment at) nogil except +
        void removeAttachment(String r, libcpp_vector[ String ] & ids, String at) nogil except +
        void removeAttachment(String r, String at) nogil except +
        void removeAllAttachments(String at) nogil except +
        void removeQualityParameter(String r, libcpp_vector[ String ] & ids) nogil except +
        void merge(QcMLFile & addendum, String setname) nogil except +
        void collectSetParameter(String setname, String qp, libcpp_vector[ String ] & ret) nogil except +
        String exportAttachment(String filename, String qpname) nogil except +
        void getRunNames(libcpp_vector[ String ] & ids) nogil except +
        bool existsRun(String filename) nogil except +
        bool existsSet(String filename) nogil except +
        void existsRunQualityParameter(String filename, String qpname, libcpp_vector[ String ] & ids) nogil except +
        void existsSetQualityParameter(String filename, String qpname, libcpp_vector[ String ] & ids) nogil except +
        void store(const String & filename) nogil except +
        void load(const String & filename) nogil except +

        void registerRun(String id_, String name) nogil except +
        void registerSet(String id_, String name, libcpp_set[ String ] & names) nogil except +
        String exportQP(String filename, String qpname) nogil except +
        String exportQPs(String filename, StringList qpnames) nogil except +
        void getRunIDs(libcpp_vector[ String ] & ids) nogil except +


cdef extern from "<OpenMS/FORMAT/QcMLFile.h>" namespace "OpenMS::QcMLFile":
    
    cdef cppclass QualityParameter "OpenMS::QcMLFile::QualityParameter":
        QualityParameter() nogil except +
        QualityParameter(QualityParameter) nogil except +
        String name
        String id
        String value
        String cvRef
        String cvAcc
        String unitRef
        String unitAcc
        String flag
        bool operator==(QualityParameter & rhs) nogil except +
        bool operator<(QualityParameter & rhs) nogil except +
        bool operator>(QualityParameter & rhs) nogil except +
        String toXMLString(UInt indentation_level) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from MetaInfoInterface cimport *

from DateTime cimport *
from SpectrumIdentification cimport *

cdef extern from "<OpenMS/METADATA/Identification.h>" namespace "OpenMS":

    cdef cppclass Identification(MetaInfoInterface):
        # wrap-inherits:
        #   MetaInfoInterface

        Identification()   nogil except +
        Identification(Identification) nogil except + # wrap-ignore


        void setCreationDate(DateTime date) nogil except +
        DateTime getCreationDate() nogil except +

        # /// sets the spectrum identifications
        void setSpectrumIdentifications(libcpp_vector[SpectrumIdentification] & ids) nogil except +

        # /// adds a spectrum identification
        void addSpectrumIdentification(SpectrumIdentification & id) nogil except +

        # /// returns the spectrum identifications stored
        libcpp_vector[SpectrumIdentification] getSpectrumIdentifications()  nogil except +




from Types cimport *
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/MATH/MISC/CubicSpline2d.h>" namespace "OpenMS":
    
    cdef cppclass CubicSpline2d "OpenMS::CubicSpline2d":

        CubicSpline2d(CubicSpline2d) nogil except + #wrap-ignore
        CubicSpline2d(libcpp_vector[ double ] x, libcpp_vector[ double ] y) nogil except +
        CubicSpline2d(libcpp_map[ double, double ] m) nogil except +

        double eval(double x) nogil except +
        double derivatives(double x, unsigned order) nogil except +




cimport numpy as np
import numpy as np








    def get_peaks(self):
        """Cython signature: numpy_vector, numpy_vector get_peaks()
        
        Will return a tuple of two numpy arrays (m/z, intensity) corresponding
        to the peaks in the MSSpectrum. Provides fast access to peaks.
        """

        cdef _MSSpectrum * spec_ = self.inst.get()

        cdef unsigned int n = spec_.size()
        cdef np.ndarray[np.float64_t, ndim=1] mzs
        mzs = np.zeros( (n,), dtype=np.float64)
        cdef np.ndarray[np.float32_t, ndim=1] intensities
        intensities = np.zeros( (n,), dtype=np.float32)
        cdef _Peak1D p

        cdef libcpp_vector[_Peak1D].iterator it = spec_.begin()
        cdef int i = 0
        while it != spec_.end():
            mzs[i] = deref(it).getMZ()
            intensities[i] = deref(it).getIntensity()
            inc(it)
            i += 1

        return mzs, intensities

    def set_peaks(self, peaks):
        """Cython signature: set_peaks((numpy_vector, numpy_vector))
        
        Takes a tuple or list of two arrays (m/z, intensity) and populates the
        MSSpectrum. The arrays can be numpy arrays (faster).
        """

        assert isinstance(peaks, (tuple, list)), "Input for set_peaks needs to be a tuple or a list of size 2 (mz and intensity vector)"
        assert len(peaks) == 2, "Input for set_peaks needs to be a tuple or a list of size 2 (mz and intensity vector)"

        mzs, intensities = peaks
        assert len(mzs) == len(intensities), "Input vectors for set_peaks need to have the same length (mz and intensity vector)"

        # Select which function to use for set_peaks:
        # If we have numpy arrays, it helps to use optimized functions
        if isinstance(mzs, np.ndarray) and isinstance(intensities, np.ndarray) and \
          mzs.dtype == np.float64 and intensities.dtype == np.float32 and \
          mzs.flags["C_CONTIGUOUS"] and intensities.flags["C_CONTIGUOUS"]  :
            self._set_peaks_fast_df(mzs, intensities)
        elif isinstance(mzs, np.ndarray) and isinstance(intensities, np.ndarray) and \
          mzs.dtype == np.float64 and intensities.dtype == np.float64 and \
          mzs.flags["C_CONTIGUOUS"] and intensities.flags["C_CONTIGUOUS"]  :
            self._set_peaks_fast_dd(mzs, intensities)
        else:
            self._set_peaks_orig(mzs, intensities)



    def _set_peaks_fast_dd(self, np.ndarray[double, ndim=1, mode="c"] data_mz not None, np.ndarray[double, ndim=1, mode="c"] data_i not None):

        cdef _MSSpectrum * spec_ = self.inst.get()

        spec_.clear(0) # empty vector , keep meta data
        spec_.reserve(<int>len(data_mz)) # allocate space for incoming data
        cdef _Peak1D p = _Peak1D()
        cdef double mz
        cdef double intensity
        cdef int N
        N = len(data_mz)

        for i in range(N):
            mz = data_mz[i]
            intensity = data_i[i]
            p.setMZ(<double>mz)
            p.setIntensity(<float>intensity)
            spec_.push_back(p)

        spec_.updateRanges()


    def _set_peaks_fast_df(self, np.ndarray[double, ndim=1, mode="c"] data_mz not None, np.ndarray[float, ndim=1, mode="c"] data_i not None):

        cdef _MSSpectrum * spec_ = self.inst.get()

        spec_.clear(0) # empty vector , keep meta data
        spec_.reserve(<int>len(data_mz)) # allocate space for incoming data
        cdef _Peak1D p = _Peak1D()
        cdef double mz
        cdef float intensity
        cdef int N
        N = len(data_mz)

        for i in range(N):
            mz = data_mz[i]
            intensity = data_i[i]
            p.setMZ(<double>mz)
            p.setIntensity(<float>intensity)
            spec_.push_back(p)

        spec_.updateRanges()


    def _set_peaks_orig(self, mzs, intensities):


        cdef _MSSpectrum * spec_ = self.inst.get()

        spec_.clear(0) # empty vector , keep meta data
        spec_.reserve(<int>len(mzs)) # allocate space for incoming data
        cdef _Peak1D p = _Peak1D()
        cdef double mz
        cdef float intensity
        cdef int N
        N = len(mzs)

        for i in range(N):
            mz = mzs[i]
            intensity = intensities[i]
            p.setMZ(<double>mz)
            p.setIntensity(<float>intensity)
            spec_.push_back(p)

        spec_.updateRanges()

    def intensityInRange(self, float mzmin, float mzmax):

        cdef int n
        cdef double I

        cdef _MSSpectrum * spec_ = self.inst.get()
        cdef int N = spec_.size()

        I = 0
        for i in range(N):
                if deref(spec_)[i].getMZ() >= mzmin:
                    break

        cdef _Peak1D * p
        for j in range(i, N):
                p = address(deref(spec_)[i])
                if p.getMZ() > mzmax:
                    break
                I += p.getIntensity()

        return I




from Types cimport *
from String cimport *
from ConsensusMap cimport *

cdef extern from "<OpenMS/FORMAT/IBSpectraFile.h>" namespace "OpenMS":
    
    cdef cppclass IBSpectraFile "OpenMS::IBSpectraFile":
        IBSpectraFile() nogil except +
        IBSpectraFile(IBSpectraFile) nogil except +
        void store(const String & filename, ConsensusMap & cm) nogil except +



from Types cimport *
from libcpp cimport bool
from String cimport *
from ModificationDefinitionsSet cimport *
# from XMLFile cimport *

cdef extern from "<OpenMS/FORMAT/XTandemInfile.h>" namespace "OpenMS":
    
    cdef cppclass XTandemInfile: # (XMLFile):
        XTandemInfile() nogil except +
        XTandemInfile(XTandemInfile) nogil except + #wrap-ignore

        void setFragmentMassTolerance(double tolerance) nogil except +
        double getFragmentMassTolerance() nogil except +

        void setPrecursorMassTolerancePlus(double tol) nogil except +
        double getPrecursorMassTolerancePlus() nogil except +
        void setPrecursorMassToleranceMinus(double tol) nogil except +
        double getPrecursorMassToleranceMinus() nogil except +

        void setPrecursorErrorType(MassType mono_isotopic) nogil except +
        MassType getPrecursorErrorType() nogil except +

        void setFragmentMassErrorUnit(ErrorUnit unit) nogil except +
        ErrorUnit getFragmentMassErrorUnit() nogil except +
        void setPrecursorMassErrorUnit(ErrorUnit unit) nogil except +
        ErrorUnit getPrecursorMassErrorUnit() nogil except +

        void setNumberOfThreads(UInt threads) nogil except +
        UInt getNumberOfThreads() nogil except +

        void setModifications(ModificationDefinitionsSet & mods) nogil except +
        ModificationDefinitionsSet  getModifications() nogil except +

        void setOutputFilename(const String & output) nogil except +
        String getOutputFilename() nogil except +
        void setInputFilename(const String & input_file) nogil except +
        String getInputFilename() nogil except +
        void setTaxonomyFilename(const String & filename) nogil except +
        String getTaxonomyFilename() nogil except +
        void setDefaultParametersFilename(const String & filename) nogil except +
        String getDefaultParametersFilename() nogil except +

        void setTaxon(const String & taxon) nogil except +
        String getTaxon() nogil except +

        void setMaxPrecursorCharge(Int max_charge) nogil except +
        Int getMaxPrecursorCharge() nogil except +

        void setNumberOfMissedCleavages(UInt missed_cleavages) nogil except +
        UInt getNumberOfMissedCleavages() nogil except +

        void setOutputResults(String result) nogil except +
        String getOutputResults() nogil except +

        void setMaxValidEValue(double value) nogil except +
        double getMaxValidEValue() nogil except +

        void setSemiCleavage(bool semi_cleavage) nogil except +
        void setAllowIsotopeError(bool allow_isotope_error) nogil except +

        # bool getNoiseSuppression() nogil except +
        # void setNoiseSuppression(bool noise_suppression) nogil except +

        void write(String filename, bool ignore_member_parameters, bool force_default_mods) nogil except +

        void setCleavageSite(String cleavage_site) nogil except +
        String getCleavageSite() nogil except +

cdef extern from "<OpenMS/FORMAT/XTandemInfile.h>" namespace "OpenMS::XTandemInfile":
    cdef enum ErrorUnit "OpenMS::XTandemInfile::ErrorUnit":
        #wrap-attach:
        #    XTandemInfile
        DALTONS
        PPM

cdef extern from "<OpenMS/FORMAT/XTandemInfile.h>" namespace "OpenMS::XTandemInfile":
    cdef enum MassType "OpenMS::XTandemInfile::MassType":
        #wrap-attach:
        #    XTandemInfile
        MONOISOTOPIC
        AVERAGE




from Types cimport *
from libcpp cimport bool
from Types cimport *
from MSSpectrum cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakShape.h>" namespace "OpenMS":
    
    cdef cppclass PeakShape "OpenMS::PeakShape":
        PeakShape() nogil except +
        PeakShape(PeakShape) nogil except +
        double height
        double mz_position
        double left_width
        double right_width
        double area
        double r_value
        double signal_to_noise
        PeakShape_Type type
        # PeakShape(double height_, double mz_position_, double left_width_, double right_width_, double area_, PeakIterator left_, PeakIterator right_, PeakShape_Type type_) nogil except +
        PeakShape(double height_, double mz_position_, double left_width_, double right_width_, double area_, PeakShape_Type type_) nogil except +
        bool operator==(PeakShape & rhs) nogil except +
        bool operator!=(PeakShape & rhs) nogil except +
        # double operator()(double x) nogil except +
        double getSymmetricMeasure() nogil except +
        double getFWHM() nogil except +
        bool iteratorsSet() nogil except +
        # PeakIterator getLeftEndpoint() nogil except +
        # void setLeftEndpoint(PeakIterator left_endpoint) nogil except +
        # PeakIterator getRightEndpoint() nogil except +
        # void setRightEndpoint(PeakIterator right_endpoint) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/PeakShape.h>" namespace "OpenMS::PeakShape":
    cdef enum PeakShape_Type "OpenMS::PeakShape::Type":
        #wrap-attach:
        #    PeakShape
        LORENTZ_PEAK
        SECH_PEAK
        UNDEFINED




from Types cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/MassDecomposer.h>" namespace "OpenMS::ims":
    
    cdef cppclass MassDecomposer[ValueType,DecompositionValueType]:
        # wrap-ignore
        # ABSTRACT class
        # no-pxd-import
        MassDecomposer(MassDecomposer) nogil except + #wrap-ignore

        bool exist(ValueType mass) nogil except +
        # decomposition_type getDecomposition(ValueType mass) nogil except +
        # decompositions_type getAllDecompositions(ValueType mass) nogil except +
        DecompositionValueType getNumberOfDecompositions(ValueType mass) nogil except +




from libcpp cimport *
from Types cimport *
from DPosition cimport *

cdef extern from "<OpenMS/KERNEL/Peak2D.h>" namespace "OpenMS":

    cdef cppclass Peak2D:
        Peak2D() nogil except +
        Peak2D(Peak2D &) nogil except +
        #Peak2D(DPosition2 &, float) nogil except +

        float getIntensity() nogil except +
        double getMZ() nogil except +
        double getRT() nogil except +
        void setMZ(double) nogil except +
        void setRT(double) nogil except +
        void setIntensity(float) nogil except +
        bool operator==(Peak2D) nogil except +
        bool operator!=(Peak2D) nogil except +
        # DPosition2 getPosition()
        # void setPosition(DPosition2 & position)

cdef extern from "<OpenMS/KERNEL/Peak2D.h>" namespace "OpenMS::Peak2D":
    
    cdef enum DimensionDescription "OpenMS::Peak2D::DimensionDescription":
        RT
        MZ
        DIMENSION



from MSExperiment cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/ChromeleonFile.h>" namespace "OpenMS":

    cdef cppclass ChromeleonFile:

        ChromeleonFile() nogil except +
        ChromeleonFile(ChromeleonFile) nogil except +

        void load(const String& filename, MSExperiment& experiment) nogil except +



from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

from MassTrace cimport *

from DefaultParamHandler cimport *
from ProgressLogger cimport *

from Types cimport *

from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/ElutionPeakDetection.h>" namespace "OpenMS":

    cdef cppclass ElutionPeakDetection(ProgressLogger, DefaultParamHandler):
        # wrap-inherits:
        #    ProgressLogger
        #    DefaultParamHandler

        ElutionPeakDetection()      nogil except +

        void detectPeaks(Kernel_MassTrace & in_,
                         libcpp_vector[Kernel_MassTrace] & out
                         ) nogil except +

        void detectPeaks(libcpp_vector[Kernel_MassTrace] & in_,
                         libcpp_vector[Kernel_MassTrace] & out
                        ) nogil except +

        void filterByPeakWidth(libcpp_vector[Kernel_MassTrace] & in_,
                               libcpp_vector[Kernel_MassTrace] & out
                              ) nogil except +

        double computeMassTraceNoise(Kernel_MassTrace &) nogil except +
        double computeMassTraceSNR(Kernel_MassTrace &) nogil except +
        double computeApexSNR(Kernel_MassTrace &) nogil except +

        void findLocalExtrema(Kernel_MassTrace & , Size & , libcpp_vector[ size_t ] & , libcpp_vector[ size_t ] & ) nogil except +

        void smoothData(Kernel_MassTrace & mt, int win_size) nogil except +




from Types cimport *
from String cimport *
from smart_ptr cimport shared_ptr
from OpenSwathDataStructures cimport *
from LightTargetedExperiment cimport *
from AASequence cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DIAScoring.h>" namespace "OpenMS":

    cdef cppclass DIAScoring(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        DIAScoring() nogil except +
        # DIAScoring(DIAScoring) nogil except + #private

        bool dia_ms1_massdiff_score(double precursor_mz, OSSpectrumPtr spectrum,
                                    double& ppm_score) nogil except + 

        void dia_ms1_isotope_scores(double precursor_mz, OSSpectrumPtr spectrum, size_t charge_state, 
                                    double& isotope_corr, double& isotope_overlap, String sum_formula) nogil except +

        # TODO automatically wrap 
        void dia_by_ion_score(OSSpectrumPtr spectrum, AASequence sequence, int charge, double & bseries_score, double & yseries_score) nogil except + # wrap-return:return(bseries_score,yseries_score) wrap-ignore

        # Dotproduct / Manhatten score with theoretical spectrum
        void score_with_isotopes(OSSpectrumPtr spectrum, libcpp_vector[LightTransition] transitions,
                                 double& dotprod, double& manhattan) nogil except +




from libcpp cimport bool
from Types cimport *
from String cimport *
from Peak1D cimport *
from EmpiricalFormula cimport *

cdef extern from "<OpenMS/CHEMISTRY/ISOTOPEDISTRIBUTION/IsotopeDistribution.h>" namespace "OpenMS":

    cdef cppclass IsotopeDistribution:        

        IsotopeDistribution() nogil except +
        IsotopeDistribution(IsotopeDistribution) nogil except + # wrap-ignore

        # overwrites the container which holds the distribution using @p distribution
        void set(libcpp_vector[ Peak1D ]& distribution) nogil except +

        void insert(double mass, float intensity) nogil except +

        libcpp_vector[Peak1D].iterator begin() nogil except +  # wrap-iter-begin:__iter__(Peak1D)
        libcpp_vector[Peak1D].iterator end()   nogil except +  # wrap-iter-end:__iter__(Peak1D)

        # returns the container which holds the distribution
        libcpp_vector[ Peak1D ]& getContainer() nogil except +

        # returns the maximal weight isotope which is stored in the distribution
        Size getMax() nogil except +

        # returns the minimal weight isotope which is stored in the distribution
        Size getMin() nogil except +

        # returns the most abundant isotope which is stored in the distribution
        Peak1D getMostAbundant() nogil except +

        # returns the size of the distribution which is the number of isotopes in the distribution
        Size size() nogil except +

        # clears the distribution and resets max isotope to 0
        void clear() nogil except +

        # renormalizes the sum of the probabilities of the isotopes to 1
        void renormalize() nogil except +

        # Trims the right side of the isotope distribution to isotopes with a significant contribution.
        void trimRight(double cutoff) nogil except +

        # Trims the left side of the isotope distribution to isotopes with a significant contribution.
        void trimLeft(double cutoff) nogil except +
        
        void merge(double, double) nogil except +

        void resize(UInt size) nogil except +
        void trimIntensities(double cutoff) nogil except +
        void sortByIntensity() nogil except +
        void sortByMass() nogil except +
        double averageMass() nogil except +

cdef extern from "<OpenMS/CHEMISTRY/ISOTOPEDISTRIBUTION/IsotopeDistribution.h>" namespace "OpenMS::IsotopeDistribution":
    
    cdef enum Sorted "OpenMS::IsotopeDistribution::Sorted":
        # wrap-attach:
        #    IsotopeDistribution
        INTENSITY
        MASS
        UNDEFINED

cdef extern from "<OpenMS/CHEMISTRY/ISOTOPEDISTRIBUTION/FineIsotopePatternGenerator.h>" namespace "OpenMS":

    cdef cppclass FineIsotopePatternGenerator:
        #
        # wrap-doc:
        #   Isotope pattern generator for fine isotope distributions.
        #   Generates isotopes until a stop condition (threshold) is reached,
        #   the lower the threshold the more isotopes are generated. The
        #   parameter use_total_prob defines whether the stop condition is
        #   interpreted as the total probability that the distribution should
        #   cover (default) or as a threshold for individual peaks. Finally,
        #   the absolute parameter specifies for individual peak thresholding
        #   if the threshold is absolute or relative.

        FineIsotopePatternGenerator() nogil except + 
        FineIsotopePatternGenerator(double threshold) nogil except +
        FineIsotopePatternGenerator(double threshold, bool use_total_prob) nogil except +
        FineIsotopePatternGenerator(double threshold, bool use_total_prob, bool absolute) nogil except +

        void setThreshold(double threshold) nogil except +
        double getThreshold() nogil except +

        void setAbsolute(bool absolute) nogil except +
        bool getAbsolute() nogil except +

        void setTotalProbability(bool total) nogil except +
        bool getTotalProbability() nogil except +

        IsotopeDistribution run(EmpiricalFormula) nogil except +

cdef extern from "<OpenMS/CHEMISTRY/ISOTOPEDISTRIBUTION/CoarseIsotopePatternGenerator.h>" namespace "OpenMS":

    cdef cppclass CoarseIsotopePatternGenerator:

        CoarseIsotopePatternGenerator() nogil except + 
        CoarseIsotopePatternGenerator(Size max_isotope) nogil except +
        CoarseIsotopePatternGenerator(Size max_isotope, bool round_masses) nogil except +

        IsotopeDistribution run(EmpiricalFormula) nogil except +

        # returns the current value of the flag to round masses to integer values (true) or return accurate masses (false)
        bool getRoundMasses() nogil except +

        # sets the round_masses_ flag to round masses to integer values (true) or return accurate masses (false)
        void setRoundMasses(bool round_masses_) nogil except +

        # returns the currently set maximum isotope
        Size getMaxIsotope() nogil except +
        
        #  sets the maximal isotope with @p max_isotope
        void setMaxIsotope(Size max_isotope) nogil except +

        # @brief Estimate Peptide Isotopedistribution from weight and number of isotopes that should be reported
        #   "Determination of Monoisotopic Masses and Ion Populations for Large Biomolecules from Resolved Isotopic Distributions"
        IsotopeDistribution estimateFromPeptideWeight(double average_weight) nogil except +

        # Estimate peptide IsotopeDistribution from average weight and exact number of sulfurs
        IsotopeDistribution estimateFromPeptideWeightAndS(double average_weight, UInt S) nogil except +

        # Estimate Nucleotide Isotopedistribution from weight
        IsotopeDistribution estimateFromRNAWeight(double average_weight) nogil except +

        # Estimate Nucleotide Isotopedistribution from weight
        IsotopeDistribution estimateFromDNAWeight(double average_weight) nogil except +

        IsotopeDistribution estimateFromWeightAndComp(double average_weight,
                                                      double C, double H,
                                                      double N, double O,
                                                      double S, double P) nogil except +

        # Estimate IsotopeDistribution from weight, exact number of sulfurs, and average remaining composition
        IsotopeDistribution estimateFromWeightAndCompAndS(double average_weight,
                                                          UInt S, double C,
                                                          double H, double N,
                                                          double O, double P) nogil except +

        # Estimate peptide fragment IsotopeDistribution from the precursor's average weight,
        # fragment's average weight, and a set of isolated precursor isotopes.
        IsotopeDistribution estimateForFragmentFromPeptideWeight(double average_weight_precursor,
                                                                 double average_weight_fragment,
                                                                 libcpp_set[ unsigned int ]& precursor_isotopes) nogil except +

        # Estimate peptide fragment IsotopeDistribution from the precursor's average weight,
        # number of sulfurs in the precursor, fragment's average weight, number of sulfurs in the fragment,
        # and a set of isolated precursor isotopes.
        IsotopeDistribution estimateForFragmentFromPeptideWeightAndS(double average_weight_precursor,
                                                                     UInt S_precursor,
                                                                     double average_weight_fragment,
                                                                     UInt S_fragment,
                                                                     libcpp_set[ unsigned int ]& precursor_isotopes) nogil except +

        # Estimate RNA fragment IsotopeDistribution from the precursor's average weight,
        # fragment's average weight, and a set of isolated precursor isotopes.
        IsotopeDistribution estimateForFragmentFromRNAWeight(double average_weight_precursor,
                                                             double average_weight_fragment,
                                                             libcpp_set[ unsigned int ]& precursor_isotopes) nogil except +

        # Estimate DNA fragment IsotopeDistribution from the precursor's average weight,
        # fragment's average weight, and a set of isolated precursor isotopes.
        IsotopeDistribution estimateForFragmentFromDNAWeight(double average_weight_precursor,
                                                             double average_weight_fragment,
                                                             libcpp_set[ unsigned int ]& precursor_isotopes) nogil except +

        # Estimate fragment IsotopeDistribution from the precursor's average weight,
        # fragment's average weight, a set of isolated precursor isotopes, and average composition
        IsotopeDistribution estimateForFragmentFromWeightAndComp(double average_weight_precursor,
                                                                 double average_weight_fragment,
                                                                 libcpp_set[ unsigned int ]& precursor_isotopes,
                                                                 double C,
                                                                 double H,
                                                                 double N,
                                                                 double O,
                                                                 double S,
                                                                 double P) nogil except +

        # Calculate isotopic distribution for a fragment molecule
        IsotopeDistribution calcFragmentIsotopeDist(IsotopeDistribution& fragment_isotope_dist,
                                                    IsotopeDistribution& comp_fragment_isotope_dist,
                                                    libcpp_set[ unsigned int ]& precursor_isotopes,
                                                    double fragment_mono_mass) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from Types cimport *

cdef extern from "<OpenMS/MATH/MISC/LinearInterpolation.h>" namespace "OpenMS::Math":
    
    cdef cppclass LinearInterpolation[KeyType,ValueType]:
        # wrap-instances:
        #   LinearInterpolation := LinearInterpolation[double, double]
        LinearInterpolation() nogil except +
        LinearInterpolation(LinearInterpolation) nogil except +
        ValueType value(KeyType arg_pos) nogil except +
        void addValue(KeyType arg_pos, ValueType arg_value) nogil except +
        ValueType derivative(KeyType arg_pos) nogil except +

        libcpp_vector[ValueType]  getData() nogil except +
        void setData(libcpp_vector[ValueType] & data) nogil except +
        bool empty() nogil except +
        KeyType key2index(KeyType pos) nogil except +
        KeyType index2key(KeyType pos) nogil except +
        KeyType  getScale() nogil except +
        void setScale(KeyType & scale) nogil except +
        KeyType  getOffset() nogil except +
        void setOffset(KeyType & offset) nogil except +
        void setMapping(KeyType & scale, KeyType & inside, KeyType & outside) nogil except +
        void setMapping(KeyType & inside_low, KeyType & outside_low, KeyType & inside_high, KeyType & outside_high) nogil except +
        KeyType  getInsideReferencePoint() nogil except +
        KeyType  getOutsideReferencePoint() nogil except +
        KeyType supportMin() nogil except +
        KeyType supportMax() nogil except +
        LinearInterpolation(KeyType scale, KeyType offset) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from ConsensusMap cimport *
from ConsensusMapNormalizerAlgorithmThreshold cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/ConsensusMapNormalizerAlgorithmMedian.h>" namespace "OpenMS::ConsensusMapNormalizerAlgorithmMedian":

    cdef enum NormalizationMethod:
        NM_SCALE,
        NM_SHIFT

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/ConsensusMapNormalizerAlgorithmMedian.h>" namespace "OpenMS":
    cdef cppclass ConsensusMapNormalizerAlgorithmMedian:

        ConsensusMapNormalizerAlgorithmMedian() nogil except +
        ConsensusMapNormalizerAlgorithmMedian(ConsensusMapNormalizerAlgorithmMedian) nogil except + #wrap-ignore

        Size computeMedians(ConsensusMap & input_map,
                            libcpp_vector[double] & medians,
                            const String & acc_filter,
                            const String & desc_filter) nogil except +

        void normalizeMaps(ConsensusMap & input_map,
                           NormalizationMethod method,
                           const String & acc_filter,
                           const String & desc_filter) nogil except +



# two empty lines are important !


cdef class SignalToNoiseEstimatorMedianChrom:

    cdef shared_ptr[_SignalToNoiseEstimatorMedian[_MSChromatogram]] inst

    def __dealloc__(self):
         self.inst.reset()


    def init(self, MSChromatogram chromatogram ):
        assert isinstance(chromatogram, MSChromatogram), 'arg chromatogram wrong type'

        self.inst.get().init((deref(chromatogram.inst.get())))

    def _init_0(self):
        self.inst = shared_ptr[_SignalToNoiseEstimatorMedian[_MSChromatogram]](new _SignalToNoiseEstimatorMedian[_MSChromatogram]())

    def _init_1(self, SignalToNoiseEstimatorMedianChrom in_0 ):
        assert isinstance(in_0, SignalToNoiseEstimatorMedianChrom), 'arg in_0 wrong type'

        self.inst = shared_ptr[_SignalToNoiseEstimatorMedian[_MSChromatogram]](new _SignalToNoiseEstimatorMedian[_MSChromatogram]((deref(in_0.inst.get()))))

    def __init__(self, *args):
        if not args:
             self._init_0(*args)
        elif (len(args)==1) and (isinstance(args[0], SignalToNoiseEstimatorMedian)):
             self._init_1(*args)
        else:
               raise Exception('can not handle type of %s' % (args,))

    def getSignalToNoise(self, index ):
        assert isinstance(index, int), 'arg data_point wrong type'

        cdef double r = self.inst.get().getSignalToNoise(index)
        return r



from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from ProteinIdentification cimport *
from PeptideIdentification cimport *
from String cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/SimpleSearchEngineAlgorithm.h>" namespace "OpenMS":
 
    cdef cppclass SimpleSearchEngineAlgorithm(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        SimpleSearchEngineAlgorithm()   nogil except +
        SimpleSearchEngineAlgorithm(SimpleSearchEngineAlgorithm) nogil except + # wrap-ignore

        void search(const String& in_mzML, 
          const String& in_db, 
          libcpp_vector[ ProteinIdentification ] & prot_ids,
          libcpp_vector[ PeptideIdentification ] & pep_ids) nogil except +




import numpy as np
from scipy.signal import fftconvolve
from time import time
import sys

def main(argv):
    if len(argv) != 1:
        print 'Usage: numpy_benchmark.py <LOG_N>'
        return;

    log_n = int(argv[0])
    n = 2**log_n

    x=np.arange(n)*(1+1j)
    y=np.arange(n)*(-1-1j)
    t1=time()
    z = fftconvolve(x,y)
    t2=time()

if __name__ == '__main__':
    main(sys.argv[1:])




from Types cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimator.h>" namespace "OpenMS":
    
    cdef cppclass SignalToNoiseEstimator[Container](DefaultParamHandler,ProgressLogger):
        # wrap-ignore
        # ABSTRACT class
        # no-pxd-import
        # wrap-inherits:
        #  DefaultParamHandler
        #  ProgressLogger
        SignalToNoiseEstimator() nogil except +
        SignalToNoiseEstimator(SignalToNoiseEstimator) nogil except +
        # void init(Container & c) nogil except +
        # double getSignalToNoise(Size index) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from ProgressLogger cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/XFDRAlgorithm.h>" namespace "OpenMS":

    cdef cppclass XFDRAlgorithm(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        XFDRAlgorithm() nogil except +
        XFDRAlgorithm(XFDRAlgorithm) nogil except + #wrap-ignore

        XFDRAlgorithm_ExitCodes run(libcpp_vector[ PeptideIdentification ]& peptide_ids,
                                    ProteinIdentification& protein_id) nogil except +

        XFDRAlgorithm_ExitCodes validateClassArguments() nogil except +

cdef extern from "<OpenMS/ANALYSIS/XLMS/XFDRAlgorithm.h>" namespace "OpenMS::XFDRAlgorithm":
    cdef enum XFDRAlgorithm_ExitCodes "OpenMS::XFDRAlgorithm::ExitCodes":
        #wrap-attach:
        #    XFDRAlgorithm
        EXECUTION_OK
        ILLEGAL_PARAMETERS
        UNEXPECTED_RESULT



from ConsensusMap cimport *
from DefaultParamHandler cimport *
from Feature cimport *
from FeatureMap cimport *
from String cimport *

from ProteinIdentification cimport *
from PeptideIdentification cimport *

from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/IDMapper.h>" namespace "OpenMS":

    cdef cppclass IDMapper(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        IDMapper() nogil except +
        IDMapper(IDMapper) nogil except +

        void annotate(MSExperiment & map_,
                      libcpp_vector[PeptideIdentification] & ids,
                      libcpp_vector[ProteinIdentification] & protein_ids,
                      bool clear_ids,
                      bool mapMS1) nogil except +

        void annotate(MSExperiment & map_,
                      FeatureMap & fmap,
                      bool clear_ids,
                      bool mapMS1) nogil except +

        void annotate(FeatureMap & map_,
                      libcpp_vector[PeptideIdentification] & ids,
                      libcpp_vector[ProteinIdentification] & protein_ids,
                      bool use_centroid_rt,
                      bool use_centroid_mz,
                      MSExperiment & spectra) nogil except +

        void annotate(ConsensusMap & map_,
                      libcpp_vector[PeptideIdentification] & ids,
                      libcpp_vector[ProteinIdentification] & protein_ids,
                      bool measure_from_subelements,
                      bool annotate_ids_with_subelements, 
                      MSExperiment & spectra) nogil except +

        IDMapper_SpectraIdentificationState mapPrecursorsToIdentifications(MSExperiment spectra,
                                                                           libcpp_vector[ PeptideIdentification ] & ids, 
                                                                           double mz_tol, double rt_tol) nogil except +

cdef extern from "<OpenMS/ANALYSIS/ID/IDMapper.h>" namespace "OpenMS::IDMapper":

    cdef enum Measure:
      MEASURE_PPM = 0,
      MEASURE_DA

cdef extern from "<OpenMS/ANALYSIS/ID/IDMapper.h>" namespace "OpenMS::IDMapper":

    cdef cppclass IDMapper_SpectraIdentificationState "OpenMS::IDMapper::SpectraIdentificationState":
        IDMapper_SpectraIdentificationState()  nogil except +
        IDMapper_SpectraIdentificationState(IDMapper_SpectraIdentificationState) nogil except + #wrap-ignore

        libcpp_vector[size_t] no_precursors
        libcpp_vector[size_t] identified
        libcpp_vector[size_t] unidentified




from Types cimport *
from MZTrafoModel cimport *
from RANSAC cimport *
from String cimport *
from CalibrationData cimport *

cdef extern from "<OpenMS/FILTERING/CALIBRATION/MZTrafoModel.h>" namespace "OpenMS":

    cdef cppclass MZTrafoModel:
        MZTrafoModel()  nogil except +
        MZTrafoModel(MZTrafoModel &) nogil except +
        MZTrafoModel(bool) nogil except +
        bool isTrained() nogil except +
        double getRT() nogil except +
        double predict(double) nogil except +
        bool train(CalibrationData, MZTrafoModel_MODELTYPE, bool, double, double) nogil except +
        bool train(libcpp_vector[double], libcpp_vector[double], libcpp_vector[double], MZTrafoModel_MODELTYPE, bool) nogil except +
        void getCoefficients(double& intercept, double& slope, double& power) nogil except +
        void setCoefficients(MZTrafoModel) nogil except +
        void setCoefficients(double, double, double) nogil except +
        String toString() nogil except +
        
cdef extern from "<OpenMS/FILTERING/CALIBRATION/MZTrafoModel.h>" namespace "OpenMS::MZTrafoModel":

    cdef enum MZTrafoModel_MODELTYPE "OpenMS::MZTrafoModel::MODELTYPE":
        LINEAR
        LINEAR_WEIGHTED
        QUADRATIC
        QUADRATIC_WEIGHTED
        SIZE_OF_MODELTYPE
    
    # static members
    # libcpp_string names_of_modeltype[] nogil except +
    MZTrafoModel_MODELTYPE nameToEnum(libcpp_string name) nogil except + # wrap-attach:MZTrafoModel
    libcpp_string enumToName(MZTrafoModel_MODELTYPE mt) nogil except + # wrap-attach:MZTrafoModel
    void setRANSACParams(RANSACParam p) nogil except + # wrap-attach:MZTrafoModel
    void setCoefficientLimits(double offset, double scale, double power) nogil except + # wrap-attach:MZTrafoModel
    bool isValidModel(MZTrafoModel& trafo) nogil except + # wrap-attach:MZTrafoModel
    Size findNearest(libcpp_vector[MZTrafoModel]& tms, double rt) nogil except + # wrap-attach:MZTrafoModel
    
